#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Run interop (cross-language) tests in parallel."""

import argparse
import atexit
import itertools
import json
import multiprocessing
import os
import re
import subprocess
import sys
import time
import uuid

import python_utils.dockerjob as dockerjob
import python_utils.jobset as jobset
import python_utils.report_utils as report_utils

# It's ok to not import because this is only necessary to upload results to BQ.
try:
    from python_utils.upload_test_results import upload_interop_results_to_bq
except ImportError as e:
    print(e)

# Docker doesn't clean up after itself, so we do it on exit.
atexit.register(lambda: subprocess.call(["stty", "echo"]))

ROOT = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), "../.."))
os.chdir(ROOT)

_DEFAULT_SERVER_PORT = 8080

_SKIP_CLIENT_COMPRESSION = [
    "client_compressed_unary",
    "client_compressed_streaming",
]

_SKIP_SERVER_COMPRESSION = [
    "server_compressed_unary",
    "server_compressed_streaming",
]

_SKIP_COMPRESSION = _SKIP_CLIENT_COMPRESSION + _SKIP_SERVER_COMPRESSION

_SKIP_ADVANCED = [
    "status_code_and_message",
    "custom_metadata",
    "unimplemented_method",
    "unimplemented_service",
]

_SKIP_SPECIAL_STATUS_MESSAGE = ["special_status_message"]

_ORCA_TEST_CASES = ["orca_per_rpc", "orca_oob"]

_GOOGLE_DEFAULT_CREDS_TEST_CASE = "google_default_credentials"

_SKIP_GOOGLE_DEFAULT_CREDS = [
    _GOOGLE_DEFAULT_CREDS_TEST_CASE,
]

_COMPUTE_ENGINE_CHANNEL_CREDS_TEST_CASE = "compute_engine_channel_credentials"

_SKIP_COMPUTE_ENGINE_CHANNEL_CREDS = [
    _COMPUTE_ENGINE_CHANNEL_CREDS_TEST_CASE,
]

_TEST_TIMEOUT = 3 * 60

# disable this test on core-based languages,
# see https://github.com/grpc/grpc/issues/9779
_SKIP_DATA_FRAME_PADDING = ["data_frame_padding"]

# report suffix "sponge_log.xml" is important for reports to get picked up by internal CI
_DOCKER_BUILD_XML_REPORT = "interop_docker_build/sponge_log.xml"
_TESTS_XML_REPORT = "interop_test/sponge_log.xml"


class CXXLanguage:
    def __init__(self):
        self.client_cwd = None
        self.server_cwd = None
        self.http2_cwd = None
        self.safename = "cxx"

    def client_cmd(self, args):
        return ["cmake/build/interop_client"] + args

    def client_cmd_http2interop(self, args):
        return ["cmake/build/http2_client"] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return ["cmake/build/interop_server"] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_DATA_FRAME_PADDING
            + _SKIP_SPECIAL_STATUS_MESSAGE
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
        )

    def unimplemented_test_cases_server(self):
        return []

    def __str__(self):
        return "c++"


class AspNetCoreLanguage:
    def __init__(self):
        self.client_cwd = "../grpc-dotnet/output/InteropTestsClient"
        self.server_cwd = "../grpc-dotnet/output/InteropTestsWebsite"
        self.safename = str(self)

    def cloud_to_prod_env(self):
        return {}

    def client_cmd(self, args):
        return ["dotnet", "exec", "InteropTestsClient.dll"] + args

    def server_cmd(self, args):
        return ["dotnet", "exec", "InteropTestsWebsite.dll"] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _ORCA_TEST_CASES

    def __str__(self):
        return "aspnetcore"


class DartLanguage:
    def __init__(self):
        self.client_cwd = "../grpc-dart/interop"
        self.server_cwd = "../grpc-dart/interop"
        self.http2_cwd = "../grpc-dart/interop"
        self.safename = str(self)

    def client_cmd(self, args):
        return ["dart", "bin/client.dart"] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return ["dart", "bin/server.dart"] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_COMPRESSION
            + _SKIP_SPECIAL_STATUS_MESSAGE
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return (
            _SKIP_COMPRESSION + _SKIP_SPECIAL_STATUS_MESSAGE + _ORCA_TEST_CASES
        )

    def __str__(self):
        return "dart"


class JavaLanguage:
    def __init__(self):
        self.client_cwd = "../grpc-java"
        self.server_cwd = "../grpc-java"
        self.http2_cwd = "../grpc-java"
        self.safename = str(self)

    def client_cmd(self, args):
        return ["./run-test-client.sh"] + args

    def client_cmd_http2interop(self, args):
        return [
            "./interop-testing/build/install/grpc-interop-testing/bin/http2-client"
        ] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return ["./run-test-server.sh"] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return []

    def unimplemented_test_cases_server(self):
        # Does not support CompressedRequest feature.
        # Only supports CompressedResponse feature for unary.
        return _SKIP_CLIENT_COMPRESSION + ["server_compressed_streaming"]

    def __str__(self):
        return "java"


class JavaOkHttpClient:
    def __init__(self):
        self.client_cwd = "../grpc-java"
        self.safename = "java"

    def client_cmd(self, args):
        return ["./run-test-client.sh", "--use_okhttp=true"] + args

    def cloud_to_prod_env(self):
        return {}

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return _SKIP_DATA_FRAME_PADDING

    def __str__(self):
        return "javaokhttp"


class GoLanguage:
    def __init__(self):
        # TODO: this relies on running inside docker
        self.client_cwd = "/go/src/google.golang.org/grpc/interop/client"
        self.server_cwd = "/go/src/google.golang.org/grpc/interop/server"
        self.http2_cwd = "/go/src/google.golang.org/grpc/interop/http2"
        self.safename = str(self)

    def client_cmd(self, args):
        return ["go", "run", "client.go"] + args

    def client_cmd_http2interop(self, args):
        return ["go", "run", "negative_http2_client.go"] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return ["go", "run", "server.go"] + args

    def global_env(self):
        return {"GO111MODULE": "on"}

    def unimplemented_test_cases(self):
        return _SKIP_COMPRESSION

    def unimplemented_test_cases_server(self):
        return _SKIP_COMPRESSION

    def __str__(self):
        return "go"


class Http2Server:
    """Represents the HTTP/2 Interop Test server

    This pretends to be a language in order to be built and run, but really it
    isn't.
    """

    def __init__(self):
        self.server_cwd = None
        self.safename = str(self)

    def server_cmd(self, args):
        return ["python test/http2_test/http2_test_server.py"]

    def cloud_to_prod_env(self):
        return {}

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _TEST_CASES
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_SPECIAL_STATUS_MESSAGE
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
        )

    def unimplemented_test_cases_server(self):
        return _TEST_CASES

    def __str__(self):
        return "http2"


class Http2Client:
    """Represents the HTTP/2 Interop Test

    This pretends to be a language in order to be built and run, but really it
    isn't.
    """

    def __init__(self):
        self.client_cwd = None
        self.safename = str(self)

    def client_cmd(self, args):
        return ["tools/http2_interop/http2_interop.test", "-test.v"] + args

    def cloud_to_prod_env(self):
        return {}

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _TEST_CASES
            + _SKIP_SPECIAL_STATUS_MESSAGE
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
        )

    def unimplemented_test_cases_server(self):
        return _TEST_CASES

    def __str__(self):
        return "http2"


class NodeLanguage:
    def __init__(self):
        self.client_cwd = "../../../../home/appuser/grpc-node"
        self.server_cwd = "../../../../home/appuser/grpc-node"
        self.safename = str(self)

    def client_cmd(self, args):
        return [
            "packages/grpc-native-core/deps/grpc/tools/run_tests/interop/with_nvm.sh",
            "node",
            "--require",
            "./test/fixtures/native_native",
            "test/interop/interop_client.js",
        ] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return [
            "packages/grpc-native-core/deps/grpc/tools/run_tests/interop/with_nvm.sh",
            "node",
            "--require",
            "./test/fixtures/native_native",
            "test/interop/interop_server.js",
        ] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _SKIP_COMPRESSION + _ORCA_TEST_CASES

    def __str__(self):
        return "node"


class NodePureJSLanguage:
    def __init__(self):
        self.client_cwd = "../../../../home/appuser/grpc-node"
        self.server_cwd = "../../../../home/appuser/grpc-node"
        self.safename = str(self)

    def client_cmd(self, args):
        return [
            "packages/grpc-native-core/deps/grpc/tools/run_tests/interop/with_nvm.sh",
            "node",
            "--require",
            "./test/fixtures/js_js",
            "test/interop/interop_client.js",
        ] + args

    def cloud_to_prod_env(self):
        return {}

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _ORCA_TEST_CASES

    def __str__(self):
        return "nodepurejs"


class PHP7Language:
    def __init__(self):
        self.client_cwd = None
        self.server_cwd = None
        self.safename = str(self)

    def client_cmd(self, args):
        return ["src/php/bin/interop_client.sh"] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return ["src/php/bin/interop_server.sh"] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_SERVER_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _SKIP_COMPRESSION + _ORCA_TEST_CASES

    def __str__(self):
        return "php7"


class ObjcLanguage:
    def __init__(self):
        self.client_cwd = "src/objective-c/tests"
        self.safename = str(self)

    def client_cmd(self, args):
        # from args, extract the server port and craft xcodebuild command out of it
        for arg in args:
            port = re.search("--server_port=(\d+)", arg)
            if port:
                portnum = port.group(1)
                cmdline = (
                    "pod install && xcodebuild -workspace Tests.xcworkspace"
                    ' -scheme InteropTestsLocalSSL -destination name="iPhone 6"'
                    " HOST_PORT_LOCALSSL=localhost:%s test" % portnum
                )
                return [cmdline]

    def cloud_to_prod_env(self):
        return {}

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        # ObjC test runs all cases with the same command. It ignores the testcase
        # cmdline argument. Here we return all but one test cases as unimplemented,
        # and depend upon ObjC test's behavior that it runs all cases even when
        # we tell it to run just one.
        return (
            _TEST_CASES[1:]
            + _SKIP_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_SPECIAL_STATUS_MESSAGE
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _SKIP_COMPRESSION + _ORCA_TEST_CASES

    def __str__(self):
        return "objc"


class RubyLanguage:
    def __init__(self):
        self.client_cwd = None
        self.server_cwd = None
        self.safename = str(self)

    def client_cmd(self, args):
        return [
            "tools/run_tests/interop/with_rvm.sh",
            "ruby",
            "src/ruby/pb/test/client.rb",
        ] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return [
            "tools/run_tests/interop/with_rvm.sh",
            "ruby",
            "src/ruby/pb/test/server.rb",
        ] + args

    def global_env(self):
        return {}

    def unimplemented_test_cases(self):
        return (
            _SKIP_SERVER_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _SKIP_COMPRESSION + _ORCA_TEST_CASES

    def __str__(self):
        return "ruby"


_PYTHON_BINARY = "py39/bin/python"


class PythonLanguage:
    def __init__(self):
        self.client_cwd = None
        self.server_cwd = None
        self.http2_cwd = None
        self.safename = str(self)

    def client_cmd(self, args):
        return [
            _PYTHON_BINARY,
            "src/python/grpcio_tests/setup.py",
            "run_interop",
            "--client",
            '--args="{}"'.format(" ".join(args)),
        ]

    def client_cmd_http2interop(self, args):
        return [
            _PYTHON_BINARY,
            "src/python/grpcio_tests/tests/http2/negative_http2_client.py",
        ] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return [
            _PYTHON_BINARY,
            "src/python/grpcio_tests/setup.py",
            "run_interop",
            "--server",
            '--args="{}"'.format(" ".join(args)),
        ]

    def global_env(self):
        return {
            "LD_LIBRARY_PATH": "{}/libs/opt".format(DOCKER_WORKDIR_ROOT),
            "PYTHONPATH": "{}/src/python/gens".format(DOCKER_WORKDIR_ROOT),
        }

    def unimplemented_test_cases(self):
        return (
            _SKIP_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _SKIP_GOOGLE_DEFAULT_CREDS
            + _SKIP_COMPUTE_ENGINE_CHANNEL_CREDS
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        return _SKIP_COMPRESSION + _ORCA_TEST_CASES

    def __str__(self):
        return "python"


class PythonAsyncIOLanguage:
    def __init__(self):
        self.client_cwd = None
        self.server_cwd = None
        self.http2_cwd = None
        self.safename = str(self)

    def client_cmd(self, args):
        return [
            _PYTHON_BINARY,
            "src/python/grpcio_tests/setup.py",
            "run_interop",
            "--use-asyncio",
            "--client",
            '--args="{}"'.format(" ".join(args)),
        ]

    def client_cmd_http2interop(self, args):
        return [
            _PYTHON_BINARY,
            "src/python/grpcio_tests/tests/http2/negative_http2_client.py",
        ] + args

    def cloud_to_prod_env(self):
        return {}

    def server_cmd(self, args):
        return [
            _PYTHON_BINARY,
            "src/python/grpcio_tests/setup.py",
            "run_interop",
            "--use-asyncio",
            "--server",
            '--args="{}"'.format(" ".join(args)),
        ]

    def global_env(self):
        return {
            "LD_LIBRARY_PATH": "{}/libs/opt".format(DOCKER_WORKDIR_ROOT),
            "PYTHONPATH": "{}/src/python/gens".format(DOCKER_WORKDIR_ROOT),
        }

    def unimplemented_test_cases(self):
        # TODO(https://github.com/grpc/grpc/issues/21707)
        return (
            _SKIP_COMPRESSION
            + _SKIP_DATA_FRAME_PADDING
            + _AUTH_TEST_CASES
            + ["timeout_on_sleeping_server"]
            + _ORCA_TEST_CASES
        )

    def unimplemented_test_cases_server(self):
        # TODO(https://github.com/grpc/grpc/issues/21749)
        return (
            _TEST_CASES
            + _AUTH_TEST_CASES
            + _HTTP2_TEST_CASES
            + _HTTP2_SERVER_TEST_CASES
        )

    def __str__(self):
        return "pythonasyncio"


_LANGUAGES = {
    "c++": CXXLanguage(),
    "aspnetcore": AspNetCoreLanguage(),
    "dart": DartLanguage(),
    "go": GoLanguage(),
    "java": JavaLanguage(),
    "javaokhttp": JavaOkHttpClient(),
    "node": NodeLanguage(),
    "nodepurejs": NodePureJSLanguage(),
    "php7": PHP7Language(),
    "objc": ObjcLanguage(),
    "ruby": RubyLanguage(),
    "python": PythonLanguage(),
    "pythonasyncio": PythonAsyncIOLanguage(),
}

# languages supported as cloud_to_cloud servers
_SERVERS = [
    "c++",
    "node",
    "aspnetcore",
    "java",
    "go",
    "ruby",
    "python",
    "dart",
    "pythonasyncio",
    "php7",
]

_TEST_CASES = [
    "large_unary",
    "empty_unary",
    "ping_pong",
    "empty_stream",
    "client_streaming",
    "server_streaming",
    "cancel_after_begin",
    "cancel_after_first_response",
    "timeout_on_sleeping_server",
    "custom_metadata",
    "status_code_and_message",
    "unimplemented_method",
    "client_compressed_unary",
    "server_compressed_unary",
    "client_compressed_streaming",
    "server_compressed_streaming",
    "unimplemented_service",
    "special_status_message",
    "orca_per_rpc",
    "orca_oob",
]

_AUTH_TEST_CASES = [
    "compute_engine_creds",
    "jwt_token_creds",
    "oauth2_auth_token",
    "per_rpc_creds",
    _GOOGLE_DEFAULT_CREDS_TEST_CASE,
    _COMPUTE_ENGINE_CHANNEL_CREDS_TEST_CASE,
]

_HTTP2_TEST_CASES = ["tls", "framing"]

_HTTP2_SERVER_TEST_CASES = [
    "rst_after_header",
    "rst_after_data",
    "rst_during_data",
    "goaway",
    "ping",
    "max_streams",
    "data_frame_padding",
    "no_df_padding_sanity_test",
]

_GRPC_CLIENT_TEST_CASES_FOR_HTTP2_SERVER_TEST_CASES = {
    "data_frame_padding": "large_unary",
    "no_df_padding_sanity_test": "large_unary",
}

_HTTP2_SERVER_TEST_CASES_THAT_USE_GRPC_CLIENTS = list(
    _GRPC_CLIENT_TEST_CASES_FOR_HTTP2_SERVER_TEST_CASES.keys()
)

_LANGUAGES_WITH_HTTP2_CLIENTS_FOR_HTTP2_SERVER_TEST_CASES = [
    "java",
    "go",
    "python",
    "c++",
]

_LANGUAGES_FOR_ALTS_TEST_CASES = ["java", "go", "c++", "python"]

_SERVERS_FOR_ALTS_TEST_CASES = ["java", "go", "c++", "python"]

_TRANSPORT_SECURITY_OPTIONS = ["tls", "alts", "insecure"]

_CUSTOM_CREDENTIALS_TYPE_OPTIONS = [
    "tls",
    "google_default_credentials",
    "compute_engine_channel_creds",
]

DOCKER_WORKDIR_ROOT = "/var/local/git/grpc"


def docker_run_cmdline(cmdline, image, docker_args=[], cwd=None, environ=None):
    """Wraps given cmdline array to create 'docker run' cmdline from it."""

    # don't use '-t' even when TTY is available, since that would break
    # the testcases generated by tools/interop_matrix/create_testcases.sh
    docker_cmdline = ["docker", "run", "-i", "--rm=true"]

    # turn environ into -e docker args
    if environ:
        for k, v in list(environ.items()):
            docker_cmdline += ["-e", "%s=%s" % (k, v)]

    # set working directory
    workdir = DOCKER_WORKDIR_ROOT
    if cwd:
        workdir = os.path.join(workdir, cwd)
    docker_cmdline += ["-w", workdir]

    docker_cmdline += docker_args + [image] + cmdline
    return docker_cmdline


def manual_cmdline(docker_cmdline, docker_image):
    """Returns docker cmdline adjusted for manual invocation."""
    print_cmdline = []
    for item in docker_cmdline:
        if item.startswith("--name="):
            continue
        if item == docker_image:
            item = "$docker_image"
        item = item.replace('"', '\\"')
        # add quotes when necessary
        if any(character.isspace() for character in item):
            item = '"%s"' % item
        print_cmdline.append(item)
    return " ".join(print_cmdline)


def write_cmdlog_maybe(cmdlog, filename):
    """Returns docker cmdline adjusted for manual invocation."""
    if cmdlog:
        with open(filename, "w") as logfile:
            logfile.write("#!/bin/bash\n")
            logfile.write("# DO NOT MODIFY\n")
            logfile.write(
                "# This file is generated by"
                " run_interop_tests.py/create_testcases.sh\n"
            )
            logfile.writelines("%s\n" % line for line in cmdlog)
        print("Command log written to file %s" % filename)


def bash_cmdline(cmdline):
    """Creates bash -c cmdline from args list."""
    # Use login shell:
    # * makes error messages clearer if executables are missing
    return ["bash", "-c", " ".join(cmdline)]


def compute_engine_creds_required(language, test_case):
    """Returns True if given test requires access to compute engine creds."""
    language = str(language)
    if test_case == "compute_engine_creds":
        return True
    if test_case == "oauth2_auth_token" and language == "c++":
        # C++ oauth2 test uses GCE creds because C++ only supports JWT
        return True
    return False


def auth_options(
    language,
    test_case,
    google_default_creds_use_key_file,
    service_account_key_file,
    default_service_account,
):
    """Returns (cmdline, env) tuple with cloud_to_prod_auth test options."""

    language = str(language)
    cmdargs = []
    env = {}

    oauth_scope_arg = "--oauth_scope=https://www.googleapis.com/auth/xapi.zoo"
    key_file_arg = "--service_account_key_file=%s" % service_account_key_file
    default_account_arg = (
        "--default_service_account=%s" % default_service_account
    )

    if test_case in ["jwt_token_creds", "per_rpc_creds", "oauth2_auth_token"]:
        if language in [
            "aspnetcore",
            "node",
            "php7",
            "python",
            "ruby",
            "nodepurejs",
        ]:
            env["GOOGLE_APPLICATION_CREDENTIALS"] = service_account_key_file
        else:
            cmdargs += [key_file_arg]

    if test_case in ["per_rpc_creds", "oauth2_auth_token"]:
        cmdargs += [oauth_scope_arg]

    if test_case == "oauth2_auth_token" and language == "c++":
        # C++ oauth2 test uses GCE creds and thus needs to know the default account
        cmdargs += [default_account_arg]

    if test_case == "compute_engine_creds":
        cmdargs += [oauth_scope_arg, default_account_arg]

    if test_case == _GOOGLE_DEFAULT_CREDS_TEST_CASE:
        if google_default_creds_use_key_file:
            env["GOOGLE_APPLICATION_CREDENTIALS"] = service_account_key_file
        cmdargs += [default_account_arg]

    if test_case == _COMPUTE_ENGINE_CHANNEL_CREDS_TEST_CASE:
        cmdargs += [default_account_arg]

    return (cmdargs, env)


def _job_kill_handler(job):
    if job._spec.container_name:
        dockerjob.docker_kill(job._spec.container_name)
        # When the job times out and we decide to kill it,
        # we need to wait a before restarting the job
        # to prevent "container name already in use" error.
        # TODO(jtattermusch): figure out a cleaner way to this.
        time.sleep(2)


def cloud_to_prod_jobspec(
    language,
    test_case,
    server_host_nickname,
    server_host,
    google_default_creds_use_key_file,
    docker_image=None,
    auth=False,
    manual_cmd_log=None,
    service_account_key_file=None,
    default_service_account=None,
    transport_security="tls",
):
    """Creates jobspec for cloud-to-prod interop test"""
    container_name = None
    cmdargs = [
        "--server_host=%s" % server_host,
        "--server_port=443",
        "--test_case=%s" % test_case,
    ]
    if transport_security == "tls":
        transport_security_options = ["--use_tls=true"]
    elif transport_security == "google_default_credentials" and str(
        language
    ) in ["c++", "go", "java", "javaokhttp"]:
        transport_security_options = [
            "--custom_credentials_type=google_default_credentials"
        ]
    elif transport_security == "compute_engine_channel_creds" and str(
        language
    ) in ["go", "java", "javaokhttp"]:
        transport_security_options = [
            "--custom_credentials_type=compute_engine_channel_creds"
        ]
    else:
        print(
            "Invalid transport security option %s in cloud_to_prod_jobspec."
            " Lang: %s" % (str(language), transport_security)
        )
        sys.exit(1)
    cmdargs = cmdargs + transport_security_options
    environ = dict(language.cloud_to_prod_env(), **language.global_env())
    if auth:
        auth_cmdargs, auth_env = auth_options(
            language,
            test_case,
            google_default_creds_use_key_file,
            service_account_key_file,
            default_service_account,
        )
        cmdargs += auth_cmdargs
        environ.update(auth_env)
    cmdline = bash_cmdline(language.client_cmd(cmdargs))
    cwd = language.client_cwd

    if docker_image:
        container_name = dockerjob.random_name(
            "interop_client_%s" % language.safename
        )
        cmdline = docker_run_cmdline(
            cmdline,
            image=docker_image,
            cwd=cwd,
            environ=environ,
            docker_args=["--net=host", "--name=%s" % container_name],
        )
        if manual_cmd_log is not None:
            if manual_cmd_log == []:
                manual_cmd_log.append(
                    'echo "Testing ${docker_image:=%s}"' % docker_image
                )
            manual_cmd_log.append(manual_cmdline(cmdline, docker_image))
        cwd = None
        environ = None

    suite_name = "cloud_to_prod_auth" if auth else "cloud_to_prod"
    test_job = jobset.JobSpec(
        cmdline=cmdline,
        cwd=cwd,
        environ=environ,
        shortname="%s:%s:%s:%s:%s"
        % (
            suite_name,
            language,
            server_host_nickname,
            test_case,
            transport_security,
        ),
        timeout_seconds=_TEST_TIMEOUT,
        flake_retries=4 if args.allow_flakes else 0,
        timeout_retries=2 if args.allow_flakes else 0,
        kill_handler=_job_kill_handler,
    )
    if docker_image:
        test_job.container_name = container_name
    return test_job


def cloud_to_cloud_jobspec(
    language,
    test_case,
    server_name,
    server_host,
    server_port,
    docker_image=None,
    transport_security="tls",
    manual_cmd_log=None,
):
    """Creates jobspec for cloud-to-cloud interop test"""
    interop_only_options = [
        "--server_host_override=foo.test.google.fr",
        "--use_test_ca=true",
    ]
    if transport_security == "tls":
        interop_only_options += ["--use_tls=true"]
    elif transport_security == "alts":
        interop_only_options += ["--use_tls=false", "--use_alts=true"]
    elif transport_security == "insecure":
        interop_only_options += ["--use_tls=false"]
    else:
        print(
            "Invalid transport security option %s in cloud_to_cloud_jobspec."
            % transport_security
        )
        sys.exit(1)

    client_test_case = test_case
    if test_case in _HTTP2_SERVER_TEST_CASES_THAT_USE_GRPC_CLIENTS:
        client_test_case = _GRPC_CLIENT_TEST_CASES_FOR_HTTP2_SERVER_TEST_CASES[
            test_case
        ]
    if client_test_case in language.unimplemented_test_cases():
        print(
            "asking client %s to run unimplemented test case %s"
            % (repr(language), client_test_case)
        )
        sys.exit(1)

    if test_case in _ORCA_TEST_CASES:
        interop_only_options += [
            '--service_config_json=\'{"loadBalancingConfig":[{"test_backend_metrics_load_balancer":{}}]}\''
        ]

    common_options = [
        "--test_case=%s" % client_test_case,
        "--server_host=%s" % server_host,
        "--server_port=%s" % server_port,
    ]

    if test_case in _HTTP2_SERVER_TEST_CASES:
        if test_case in _HTTP2_SERVER_TEST_CASES_THAT_USE_GRPC_CLIENTS:
            client_options = interop_only_options + common_options
            cmdline = bash_cmdline(language.client_cmd(client_options))
            cwd = language.client_cwd
        else:
            cmdline = bash_cmdline(
                language.client_cmd_http2interop(common_options)
            )
            cwd = language.http2_cwd
    else:
        cmdline = bash_cmdline(
            language.client_cmd(common_options + interop_only_options)
        )
        cwd = language.client_cwd

    environ = language.global_env()
    if docker_image and language.safename != "objc":
        # we can't run client in docker for objc.
        container_name = dockerjob.random_name(
            "interop_client_%s" % language.safename
        )
        cmdline = docker_run_cmdline(
            cmdline,
            image=docker_image,
            environ=environ,
            cwd=cwd,
            docker_args=["--net=host", "--name=%s" % container_name],
        )
        if manual_cmd_log is not None:
            if manual_cmd_log == []:
                manual_cmd_log.append(
                    'echo "Testing ${docker_image:=%s}"' % docker_image
                )
            manual_cmd_log.append(manual_cmdline(cmdline, docker_image))
        cwd = None

    test_job = jobset.JobSpec(
        cmdline=cmdline,
        cwd=cwd,
        environ=environ,
        shortname="cloud_to_cloud:%s:%s_server:%s:%s"
        % (language, server_name, test_case, transport_security),
        timeout_seconds=_TEST_TIMEOUT,
        flake_retries=4 if args.allow_flakes else 0,
        timeout_retries=2 if args.allow_flakes else 0,
        kill_handler=_job_kill_handler,
    )
    if docker_image:
        test_job.container_name = container_name
    return test_job


def server_jobspec(
    language, docker_image, transport_security="tls", manual_cmd_log=None
):
    """Create jobspec for running a server"""
    container_name = dockerjob.random_name(
        "interop_server_%s" % language.safename
    )
    server_cmd = ["--port=%s" % _DEFAULT_SERVER_PORT]
    if transport_security == "tls":
        server_cmd += ["--use_tls=true"]
    elif transport_security == "alts":
        server_cmd += ["--use_tls=false", "--use_alts=true"]
    elif transport_security == "insecure":
        server_cmd += ["--use_tls=false"]
    else:
        print(
            "Invalid transport security option %s in server_jobspec."
            % transport_security
        )
        sys.exit(1)
    cmdline = bash_cmdline(language.server_cmd(server_cmd))
    environ = language.global_env()
    docker_args = ["--name=%s" % container_name]
    if language.safename == "http2":
        # we are running the http2 interop server. Open next N ports beginning
        # with the server port. These ports are used for http2 interop test
        # (one test case per port).
        docker_args += list(
            itertools.chain.from_iterable(
                ("-p", str(_DEFAULT_SERVER_PORT + i))
                for i in range(len(_HTTP2_SERVER_TEST_CASES))
            )
        )
        # Enable docker's healthcheck mechanism.
        # This runs a Python script inside the container every second. The script
        # pings the http2 server to verify it is ready. The 'health-retries' flag
        # specifies the number of consecutive failures before docker will report
        # the container's status as 'unhealthy'. Prior to the first 'health_retries'
        # failures or the first success, the status will be 'starting'. 'docker ps'
        # or 'docker inspect' can be used to see the health of the container on the
        # command line.
        docker_args += [
            "--health-cmd=python test/http2_test/http2_server_health_check.py "
            "--server_host=%s --server_port=%d"
            % ("localhost", _DEFAULT_SERVER_PORT),
            "--health-interval=1s",
            "--health-retries=5",
            "--health-timeout=10s",
        ]

    else:
        docker_args += ["-p", str(_DEFAULT_SERVER_PORT)]

    docker_cmdline = docker_run_cmdline(
        cmdline,
        image=docker_image,
        cwd=language.server_cwd,
        environ=environ,
        docker_args=docker_args,
    )
    if manual_cmd_log is not None:
        if manual_cmd_log == []:
            manual_cmd_log.append(
                'echo "Testing ${docker_image:=%s}"' % docker_image
            )
        manual_cmd_log.append(manual_cmdline(docker_cmdline, docker_image))
    server_job = jobset.JobSpec(
        cmdline=docker_cmdline,
        environ=environ,
        shortname="interop_server_%s" % language,
        timeout_seconds=30 * 60,
    )
    server_job.container_name = container_name
    return server_job


def build_interop_image_jobspec(language, tag=None):
    """Creates jobspec for building interop docker image for a language"""
    if not tag:
        tag = "grpc_interop_%s:%s" % (language.safename, uuid.uuid4())
    env = {
        "INTEROP_IMAGE": tag,
        "BASE_NAME": "grpc_interop_%s" % language.safename,
    }
    build_job = jobset.JobSpec(
        cmdline=["tools/run_tests/dockerize/build_interop_image.sh"],
        environ=env,
        shortname="build_docker_%s" % (language),
        timeout_seconds=75 * 60,
    )
    build_job.tag = tag
    return build_job


def aggregate_http2_results(stdout):
    match = re.search(r'\{"cases[^\]]*\]\}', stdout)
    if not match:
        return None

    results = json.loads(match.group(0))
    skipped = 0
    passed = 0
    failed = 0
    failed_cases = []
    for case in results["cases"]:
        if case.get("skipped", False):
            skipped += 1
        else:
            if case.get("passed", False):
                passed += 1
            else:
                failed += 1
                failed_cases.append(case.get("name", "NONAME"))
    return {
        "passed": passed,
        "failed": failed,
        "skipped": skipped,
        "failed_cases": ", ".join(failed_cases),
        "percent": 1.0 * passed / (passed + failed),
    }


# A dictionary of prod servers to test against.
# See go/grpc-interop-tests (internal-only) for details.
prod_servers = {
    "default": "grpc-test.sandbox.googleapis.com",
    "gateway_v4": "grpc-test4.sandbox.googleapis.com",
}

argp = argparse.ArgumentParser(description="Run interop tests.")
argp.add_argument(
    "-l",
    "--language",
    choices=["all"] + sorted(_LANGUAGES),
    nargs="+",
    default=["all"],
    help="Clients to run. Objc client can be only run on OSX.",
)
argp.add_argument("-j", "--jobs", default=multiprocessing.cpu_count(), type=int)
argp.add_argument(
    "--cloud_to_prod",
    default=False,
    action="store_const",
    const=True,
    help="Run cloud_to_prod tests.",
)
argp.add_argument(
    "--cloud_to_prod_auth",
    default=False,
    action="store_const",
    const=True,
    help="Run cloud_to_prod_auth tests.",
)
argp.add_argument(
    "--google_default_creds_use_key_file",
    default=False,
    action="store_const",
    const=True,
    help=(
        "Whether or not we should use a key file for the "
        "google_default_credentials test case, e.g. by "
        "setting env var GOOGLE_APPLICATION_CREDENTIALS."
    ),
)
argp.add_argument(
    "--prod_servers",
    choices=list(prod_servers.keys()),
    default=["default"],
    nargs="+",
    help=(
        "The servers to run cloud_to_prod and cloud_to_prod_auth tests against."
    ),
)
argp.add_argument(
    "-s",
    "--server",
    choices=["all"] + sorted(_SERVERS),
    nargs="+",
    help="Run cloud_to_cloud servers in a separate docker "
    + "image. Servers can only be started automatically if "
    + "--use_docker option is enabled.",
    default=[],
)
argp.add_argument(
    "--override_server",
    action="append",
    type=lambda kv: kv.split("="),
    help=(
        "Use servername=HOST:PORT to explicitly specify a server. E.g."
        " csharp=localhost:50000"
    ),
    default=[],
)
# TODO(jtattermusch): the default service_account_key_file only works when --use_docker is used.
argp.add_argument(
    "--service_account_key_file",
    type=str,
    help="The service account key file to use for some auth interop tests.",
    default="/root/service_account/grpc-testing-ebe7c1ac7381.json",
)
argp.add_argument(
    "--default_service_account",
    type=str,
    help=(
        "Default GCE service account email to use for some auth interop tests."
    ),
    default="830293263384-compute@developer.gserviceaccount.com",
)
argp.add_argument(
    "-t",
    "--travis",
    default=False,
    action="store_const",
    const=True,
    help=(
        "When set, indicates that the script is running on CI (= not locally)."
    ),
)
argp.add_argument(
    "-v", "--verbose", default=False, action="store_const", const=True
)
argp.add_argument(
    "--use_docker",
    default=False,
    action="store_const",
    const=True,
    help="Run all the interop tests under docker. That provides "
    + "additional isolation and prevents the need to install "
    + "language specific prerequisites. Only available on Linux.",
)
argp.add_argument(
    "--allow_flakes",
    default=False,
    action="store_const",
    const=True,
    help=(
        "Allow flaky tests to show as passing (re-runs failed tests up to five"
        " times)"
    ),
)
argp.add_argument(
    "--manual_run",
    default=False,
    action="store_const",
    const=True,
    help="Prepare things for running interop tests manually. "
    + "Preserve docker images after building them and skip "
    "actually running the tests. Only print commands to run by " + "hand.",
)
argp.add_argument(
    "--http2_interop",
    default=False,
    action="store_const",
    const=True,
    help="Enable HTTP/2 client edge case testing. (Bad client, good server)",
)
argp.add_argument(
    "--http2_server_interop",
    default=False,
    action="store_const",
    const=True,
    help=(
        "Enable HTTP/2 server edge case testing. (Includes positive and"
        " negative tests"
    ),
)
argp.add_argument(
    "--transport_security",
    choices=_TRANSPORT_SECURITY_OPTIONS,
    default="tls",
    type=str,
    nargs="?",
    const=True,
    help="Which transport security mechanism to use.",
)
argp.add_argument(
    "--custom_credentials_type",
    choices=_CUSTOM_CREDENTIALS_TYPE_OPTIONS,
    default=_CUSTOM_CREDENTIALS_TYPE_OPTIONS,
    nargs="+",
    help=(
        "Credential types to test in the cloud_to_prod setup. Default is to"
        " test with all creds types possible."
    ),
)
argp.add_argument(
    "--skip_compute_engine_creds",
    default=False,
    action="store_const",
    const=True,
    help="Skip auth tests requiring access to compute engine credentials.",
)
argp.add_argument(
    "--bq_result_table",
    default="",
    type=str,
    nargs="?",
    help="Upload test results to a specified BQ table.",
)
args = argp.parse_args()

servers = set(
    s
    for s in itertools.chain.from_iterable(
        _SERVERS if x == "all" else [x] for x in args.server
    )
)
# ALTS servers are only available for certain languages.
if args.transport_security == "alts":
    servers = servers.intersection(_SERVERS_FOR_ALTS_TEST_CASES)

if args.use_docker:
    if not args.travis:
        print("Seen --use_docker flag, will run interop tests under docker.")
        print("")
        print(
            "IMPORTANT: The changes you are testing need to be locally"
            " committed"
        )
        print(
            "because only the committed changes in the current branch will be"
        )
        print("copied to the docker environment.")
        time.sleep(5)

if args.manual_run and not args.use_docker:
    print("--manual_run is only supported with --use_docker option enabled.")
    sys.exit(1)

if not args.use_docker and servers:
    print(
        "Running interop servers is only supported with --use_docker option"
        " enabled."
    )
    sys.exit(1)

# we want to include everything but objc in 'all'
# because objc won't run on non-mac platforms
all_but_objc = set(_LANGUAGES) - set(["objc"])
languages = set(
    _LANGUAGES[l]
    for l in itertools.chain.from_iterable(
        all_but_objc if x == "all" else [x] for x in args.language
    )
)
# ALTS interop clients are only available for certain languages.
if args.transport_security == "alts":
    alts_languages = set(_LANGUAGES[l] for l in _LANGUAGES_FOR_ALTS_TEST_CASES)
    languages = languages.intersection(alts_languages)

languages_http2_clients_for_http2_server_interop = set()
if args.http2_server_interop:
    languages_http2_clients_for_http2_server_interop = set(
        _LANGUAGES[l]
        for l in _LANGUAGES_WITH_HTTP2_CLIENTS_FOR_HTTP2_SERVER_TEST_CASES
        if "all" in args.language or l in args.language
    )

http2Interop = Http2Client() if args.http2_interop else None
http2InteropServer = Http2Server() if args.http2_server_interop else None

docker_images = {}
if args.use_docker:
    # languages for which to build docker images
    languages_to_build = set(
        _LANGUAGES[k]
        for k in set([str(l) for l in languages] + [s for s in servers])
    )
    languages_to_build = (
        languages_to_build | languages_http2_clients_for_http2_server_interop
    )

    if args.http2_interop:
        languages_to_build.add(http2Interop)

    if args.http2_server_interop:
        languages_to_build.add(http2InteropServer)

    build_jobs = []
    for l in languages_to_build:
        if str(l) == "objc":
            # we don't need to build a docker image for objc
            continue
        job = build_interop_image_jobspec(l)
        docker_images[str(l)] = job.tag
        build_jobs.append(job)

    if build_jobs:
        jobset.message(
            "START", "Building interop docker images.", do_newline=True
        )
        if args.verbose:
            print("Jobs to run: \n%s\n" % "\n".join(str(j) for j in build_jobs))

        num_failures, build_resultset = jobset.run(
            build_jobs, newline_on_success=True, maxjobs=args.jobs
        )

        report_utils.render_junit_xml_report(
            build_resultset, _DOCKER_BUILD_XML_REPORT
        )

        if num_failures == 0:
            jobset.message(
                "SUCCESS",
                "All docker images built successfully.",
                do_newline=True,
            )
        else:
            jobset.message(
                "FAILED",
                "Failed to build interop docker images.",
                do_newline=True,
            )
            for image in docker_images.values():
                dockerjob.remove_image(image, skip_nonexistent=True)
            sys.exit(1)

server_manual_cmd_log = [] if args.manual_run else None
client_manual_cmd_log = [] if args.manual_run else None

# Start interop servers.
server_jobs = {}
server_addresses = {}
try:
    for s in servers:
        lang = str(s)
        spec = server_jobspec(
            _LANGUAGES[lang],
            docker_images.get(lang),
            args.transport_security,
            manual_cmd_log=server_manual_cmd_log,
        )
        if not args.manual_run:
            job = dockerjob.DockerJob(spec)
            server_jobs[lang] = job
            server_addresses[lang] = (
                "localhost",
                job.mapped_port(_DEFAULT_SERVER_PORT),
            )
        else:
            # don't run the server, set server port to a placeholder value
            server_addresses[lang] = ("localhost", "${SERVER_PORT}")

    http2_server_job = None
    if args.http2_server_interop:
        # launch a HTTP2 server emulator that creates edge cases
        lang = str(http2InteropServer)
        spec = server_jobspec(
            http2InteropServer,
            docker_images.get(lang),
            manual_cmd_log=server_manual_cmd_log,
        )
        if not args.manual_run:
            http2_server_job = dockerjob.DockerJob(spec)
            server_jobs[lang] = http2_server_job
        else:
            # don't run the server, set server port to a placeholder value
            server_addresses[lang] = ("localhost", "${SERVER_PORT}")

    jobs = []
    if args.cloud_to_prod:
        if args.transport_security not in ["tls"]:
            print("TLS is always enabled for cloud_to_prod scenarios.")
        for server_host_nickname in args.prod_servers:
            for language in languages:
                for test_case in _TEST_CASES:
                    if not test_case in language.unimplemented_test_cases():
                        if (
                            not test_case
                            in _SKIP_ADVANCED
                            + _SKIP_COMPRESSION
                            + _SKIP_SPECIAL_STATUS_MESSAGE
                            + _ORCA_TEST_CASES
                        ):
                            for (
                                transport_security
                            ) in args.custom_credentials_type:
                                # google_default_credentials not yet supported by all languages
                                if (
                                    transport_security
                                    == "google_default_credentials"
                                    and str(language)
                                    not in ["c++", "go", "java", "javaokhttp"]
                                ):
                                    continue
                                # compute_engine_channel_creds not yet supported by all languages
                                if (
                                    transport_security
                                    == "compute_engine_channel_creds"
                                    and str(language)
                                    not in ["go", "java", "javaokhttp"]
                                ):
                                    continue
                                test_job = cloud_to_prod_jobspec(
                                    language,
                                    test_case,
                                    server_host_nickname,
                                    prod_servers[server_host_nickname],
                                    google_default_creds_use_key_file=args.google_default_creds_use_key_file,
                                    docker_image=docker_images.get(
                                        str(language)
                                    ),
                                    manual_cmd_log=client_manual_cmd_log,
                                    service_account_key_file=args.service_account_key_file,
                                    default_service_account=args.default_service_account,
                                    transport_security=transport_security,
                                )
                                jobs.append(test_job)
            if args.http2_interop:
                for test_case in _HTTP2_TEST_CASES:
                    test_job = cloud_to_prod_jobspec(
                        http2Interop,
                        test_case,
                        server_host_nickname,
                        prod_servers[server_host_nickname],
                        google_default_creds_use_key_file=args.google_default_creds_use_key_file,
                        docker_image=docker_images.get(str(http2Interop)),
                        manual_cmd_log=client_manual_cmd_log,
                        service_account_key_file=args.service_account_key_file,
                        default_service_account=args.default_service_account,
                        transport_security=args.transport_security,
                    )
                    jobs.append(test_job)

    if args.cloud_to_prod_auth:
        if args.transport_security not in ["tls"]:
            print("TLS is always enabled for cloud_to_prod scenarios.")
        for server_host_nickname in args.prod_servers:
            for language in languages:
                for test_case in _AUTH_TEST_CASES:
                    if (
                        not args.skip_compute_engine_creds
                        or not compute_engine_creds_required(
                            language, test_case
                        )
                    ):
                        if not test_case in language.unimplemented_test_cases():
                            if test_case == _GOOGLE_DEFAULT_CREDS_TEST_CASE:
                                transport_security = (
                                    "google_default_credentials"
                                )
                            elif (
                                test_case
                                == _COMPUTE_ENGINE_CHANNEL_CREDS_TEST_CASE
                            ):
                                transport_security = (
                                    "compute_engine_channel_creds"
                                )
                            else:
                                transport_security = "tls"
                            if (
                                transport_security
                                not in args.custom_credentials_type
                            ):
                                continue
                            test_job = cloud_to_prod_jobspec(
                                language,
                                test_case,
                                server_host_nickname,
                                prod_servers[server_host_nickname],
                                google_default_creds_use_key_file=args.google_default_creds_use_key_file,
                                docker_image=docker_images.get(str(language)),
                                auth=True,
                                manual_cmd_log=client_manual_cmd_log,
                                service_account_key_file=args.service_account_key_file,
                                default_service_account=args.default_service_account,
                                transport_security=transport_security,
                            )
                            jobs.append(test_job)
    for server in args.override_server:
        server_name = server[0]
        (server_host, server_port) = server[1].split(":")
        server_addresses[server_name] = (server_host, server_port)

    for server_name, server_address in list(server_addresses.items()):
        (server_host, server_port) = server_address
        server_language = _LANGUAGES.get(server_name, None)
        skip_server = []  # test cases unimplemented by server
        if server_language:
            skip_server = server_language.unimplemented_test_cases_server()
        for language in languages:
            for test_case in _TEST_CASES:
                if not test_case in language.unimplemented_test_cases():
                    if not test_case in skip_server:
                        test_job = cloud_to_cloud_jobspec(
                            language,
                            test_case,
                            server_name,
                            server_host,
                            server_port,
                            docker_image=docker_images.get(str(language)),
                            transport_security=args.transport_security,
                            manual_cmd_log=client_manual_cmd_log,
                        )
                        jobs.append(test_job)

        if args.http2_interop:
            for test_case in _HTTP2_TEST_CASES:
                if server_name == "go":
                    # TODO(carl-mastrangelo): Reenable after https://github.com/grpc/grpc-go/issues/434
                    continue
                test_job = cloud_to_cloud_jobspec(
                    http2Interop,
                    test_case,
                    server_name,
                    server_host,
                    server_port,
                    docker_image=docker_images.get(str(http2Interop)),
                    transport_security=args.transport_security,
                    manual_cmd_log=client_manual_cmd_log,
                )
                jobs.append(test_job)

    if args.http2_server_interop:
        if not args.manual_run:
            http2_server_job.wait_for_healthy(timeout_seconds=600)
        for language in languages_http2_clients_for_http2_server_interop:
            for test_case in set(_HTTP2_SERVER_TEST_CASES) - set(
                _HTTP2_SERVER_TEST_CASES_THAT_USE_GRPC_CLIENTS
            ):
                offset = sorted(_HTTP2_SERVER_TEST_CASES).index(test_case)
                server_port = _DEFAULT_SERVER_PORT + offset
                if not args.manual_run:
                    server_port = http2_server_job.mapped_port(server_port)
                test_job = cloud_to_cloud_jobspec(
                    language,
                    test_case,
                    str(http2InteropServer),
                    "localhost",
                    server_port,
                    docker_image=docker_images.get(str(language)),
                    manual_cmd_log=client_manual_cmd_log,
                )
                jobs.append(test_job)
        for language in languages:
            # HTTP2_SERVER_TEST_CASES_THAT_USE_GRPC_CLIENTS is a subset of
            # HTTP_SERVER_TEST_CASES, in which clients use their gRPC interop clients rather
            # than specialized http2 clients, reusing existing test implementations.
            # For example, in the "data_frame_padding" test, use language's gRPC
            # interop clients and make them think that they're running "large_unary"
            # test case. This avoids implementing a new test case in each language.
            for test_case in _HTTP2_SERVER_TEST_CASES_THAT_USE_GRPC_CLIENTS:
                if test_case not in language.unimplemented_test_cases():
                    offset = sorted(_HTTP2_SERVER_TEST_CASES).index(test_case)
                    server_port = _DEFAULT_SERVER_PORT + offset
                    if not args.manual_run:
                        server_port = http2_server_job.mapped_port(server_port)
                    if args.transport_security != "insecure":
                        print(
                            "Creating grpc client to http2 server test case "
                            "with insecure connection, even though "
                            "args.transport_security is not insecure. Http2 "
                            "test server only supports insecure connections."
                        )
                    test_job = cloud_to_cloud_jobspec(
                        language,
                        test_case,
                        str(http2InteropServer),
                        "localhost",
                        server_port,
                        docker_image=docker_images.get(str(language)),
                        transport_security="insecure",
                        manual_cmd_log=client_manual_cmd_log,
                    )
                    jobs.append(test_job)

    if not jobs:
        print("No jobs to run.")
        for image in docker_images.values():
            dockerjob.remove_image(image, skip_nonexistent=True)
        sys.exit(1)

    if args.manual_run:
        print("All tests will skipped --manual_run option is active.")

    if args.verbose:
        print("Jobs to run: \n%s\n" % "\n".join(str(job) for job in jobs))

    num_failures, resultset = jobset.run(
        jobs,
        newline_on_success=True,
        maxjobs=args.jobs,
        skip_jobs=args.manual_run,
    )
    if args.bq_result_table and resultset:
        upload_interop_results_to_bq(resultset, args.bq_result_table)
    if num_failures:
        jobset.message("FAILED", "Some tests failed", do_newline=True)
    else:
        jobset.message("SUCCESS", "All tests passed", do_newline=True)

    write_cmdlog_maybe(server_manual_cmd_log, "interop_server_cmds.sh")
    write_cmdlog_maybe(client_manual_cmd_log, "interop_client_cmds.sh")

    report_utils.render_junit_xml_report(resultset, _TESTS_XML_REPORT)

    for name, job in list(resultset.items()):
        if "http2" in name:
            job[0].http2results = aggregate_http2_results(job[0].message)

    http2_server_test_cases = (
        _HTTP2_SERVER_TEST_CASES if args.http2_server_interop else []
    )

    if num_failures:
        sys.exit(1)
    else:
        sys.exit(0)
finally:
    # Check if servers are still running.
    for server, job in list(server_jobs.items()):
        if not job.is_running():
            print('Server "%s" has exited prematurely.' % server)

    dockerjob.finish_jobs([j for j in server_jobs.values()])

    for image in docker_images.values():
        if not args.manual_run:
            print("Removing docker image %s" % image)
            dockerjob.remove_image(image)
        else:
            print("Preserving docker image: %s" % image)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generates grpc-prefixed packages using template renderer.

To use this script, please use 3.9+ interpreter. This script is work-directory
agnostic. A quick executable command:

    python3 tools/distrib/python/grpc_prefixed/generate.py
"""

import dataclasses
import datetime
import logging
import os
import shutil
import subprocess
import sys

import jinja2

WORK_PATH = os.path.realpath(os.path.dirname(__file__))
LICENSE = os.path.join(WORK_PATH, "../../../../LICENSE")
BUILD_PATH = os.path.join(WORK_PATH, "build")
DIST_PATH = os.path.join(WORK_PATH, "dist")

env = jinja2.Environment(
    loader=jinja2.FileSystemLoader(os.path.join(WORK_PATH, "templates"))
)

LOGGER = logging.getLogger(__name__)
POPEN_TIMEOUT_S = datetime.timedelta(minutes=1).total_seconds()


@dataclasses.dataclass
class PackageMeta:
    """Meta-info of a PyPI package."""

    name: str
    name_long: str
    destination_package: str
    version: str = "1.0.0"


def clean() -> None:
    try:
        shutil.rmtree(BUILD_PATH)
    except FileNotFoundError:
        pass

    try:
        shutil.rmtree(DIST_PATH)
    except FileNotFoundError:
        pass


def generate_package(meta: PackageMeta) -> None:
    # Makes package directory
    package_path = os.path.join(BUILD_PATH, meta.name)
    os.makedirs(package_path, exist_ok=True)

    # Copy license
    shutil.copyfile(LICENSE, os.path.join(package_path, "LICENSE"))

    # Generates source code
    for template_name in env.list_templates():
        template = env.get_template(template_name)
        with open(
            os.path.join(package_path, template_name.replace(".template", "")),
            "w",
        ) as f:
            f.write(template.render(dataclasses.asdict(meta)))

    # Creates wheel
    job = subprocess.Popen(
        [
            sys.executable,
            os.path.join(package_path, "setup.py"),
            "sdist",
            "--dist-dir",
            DIST_PATH,
        ],
        cwd=package_path,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    )
    outs, _ = job.communicate(timeout=POPEN_TIMEOUT_S)

    # Logs result
    if job.returncode != 0:
        LOGGER.error("Wheel creation failed with %d", job.returncode)
        LOGGER.error(outs)
    else:
        LOGGER.info("Package <%s> generated", meta.name)


def main():
    clean()

    generate_package(
        PackageMeta(
            name="grpc",
            name_long="gRPC Python",
            destination_package="grpcio",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-status",
            name_long="gRPC Rich Error Status",
            destination_package="grpcio-status",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-channelz",
            name_long="gRPC Channel Tracing",
            destination_package="grpcio-channelz",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-tools",
            name_long="ProtoBuf Code Generator",
            destination_package="grpcio-tools",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-reflection",
            name_long="gRPC Reflection",
            destination_package="grpcio-reflection",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-testing",
            name_long="gRPC Testing Utility",
            destination_package="grpcio-testing",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-health-checking",
            name_long="gRPC Health Checking",
            destination_package="grpcio-health-checking",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-csds",
            name_long="gRPC Client Status Discovery Service",
            destination_package="grpcio-csds",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-admin",
            name_long="gRPC Admin Interface",
            destination_package="grpcio-admin",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-observability",
            name_long="gRPC Python observability package",
            destination_package="grpcio-observability",
        )
    )

    generate_package(
        PackageMeta(
            name="grpc-csm-observability",
            name_long="gRPC Python CSM observability package",
            destination_package="grpcio-csm-observability",
        )
    )


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/config/v3alpha/kv_store_xds_delegate_config.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/config/v3alpha/kv_store_xds_delegate_config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.common.key_value.v3 import config_pb2 as envoy_dot_config_dot_common_dot_key__value_dot_v3_dot_config__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nJcontrib/envoy/extensions/config/v3alpha/kv_store_xds_delegate_config.proto\x12\x1f\x65nvoy.extensions.config.v3alpha\x1a-envoy/config/common/key_value/v3/config.proto\x1a\x1dudpa/annotations/status.proto\"w\n\x1eKeyValueStoreXdsDelegateConfig\x12U\n\x16key_value_store_config\x18\x01 \x01(\x0b\x32\x35.envoy.config.common.key_value.v3.KeyValueStoreConfigB\xa8\x01\n-io.envoyproxy.envoy.extensions.config.v3alphaB\x1dKvStoreXdsDelegateConfigProtoP\x01ZNgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/config/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.config.v3alpha.kv_store_xds_delegate_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n-io.envoyproxy.envoy.extensions.config.v3alphaB\035KvStoreXdsDelegateConfigProtoP\001ZNgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/config/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_KEYVALUESTOREXDSDELEGATECONFIG']._serialized_start=189
  _globals['_KEYVALUESTOREXDSDELEGATECONFIG']._serialized_end=308
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Definition of targets run distribution package tests."""

import datetime
import os.path
import sys

sys.path.insert(0, os.path.abspath(".."))
import python_utils.jobset as jobset

REPORT_BASE_PATH = os.getenv("GRPC_TEST_REPORT_BASE_DIR", os.path.abspath("."))


def create_docker_jobspec(
    name,
    dockerfile_dir,
    shell_command,
    environ={},
    flake_retries=0,
    timeout_retries=0,
    copy_rel_path=None,
    timeout_seconds=30 * 60,
):
    """Creates jobspec for a task running under docker."""
    environ = environ.copy()
    # the entire repo will be cloned if copy_rel_path is not set.
    if copy_rel_path:
        environ["RELATIVE_COPY_PATH"] = copy_rel_path

    docker_args = []
    for k, v in list(environ.items()):
        docker_args += ["-e", "%s=%s" % (k, v)]
    docker_env = {
        "DOCKERFILE_DIR": dockerfile_dir,
        "DOCKER_RUN_SCRIPT": "tools/run_tests/dockerize/docker_run.sh",
        "DOCKER_RUN_SCRIPT_COMMAND": shell_command,
    }
    jobspec = jobset.JobSpec(
        cmdline=["tools/run_tests/dockerize/build_and_run_docker.sh"]
        + docker_args,
        environ=docker_env,
        shortname="distribtest.%s" % (name),
        timeout_seconds=timeout_seconds,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries,
        logfilename=os.path.abspath(
            f"{REPORT_BASE_PATH}/reports/distribtest.{name}.log"
        ),
    )
    return jobspec


def create_jobspec(
    name,
    cmdline,
    environ=None,
    shell=False,
    flake_retries=0,
    timeout_retries=0,
    use_workspace=False,
    timeout_seconds=10 * 60,
):
    """Creates jobspec."""
    environ = environ.copy()
    if use_workspace:
        environ["WORKSPACE_NAME"] = "workspace_%s" % name
        cmdline = [
            "bash",
            "tools/run_tests/artifacts/run_in_workspace.sh",
        ] + cmdline
    jobspec = jobset.JobSpec(
        cmdline=cmdline,
        environ=environ,
        shortname="distribtest.%s" % (name),
        timeout_seconds=timeout_seconds,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries,
        shell=shell,
        logfilename=os.path.abspath(
            f"{REPORT_BASE_PATH}/reports/distribtest.{name}.log"
        ),
    )
    return jobspec


class CSharpDistribTest:
    """Tests C# NuGet package"""

    def __init__(
        self,
        platform,
        arch,
        docker_suffix=None,
        use_dotnet_cli=False,
        presubmit=False,
    ):
        self.name = "csharp_%s_%s" % (platform, arch)
        self.platform = platform
        self.arch = arch
        self.docker_suffix = docker_suffix
        self.labels = ["distribtest", "csharp", platform, arch]
        if presubmit:
            self.labels.append("presubmit")
        self.script_suffix = ""
        if docker_suffix:
            self.name += "_%s" % docker_suffix
            self.labels.append(docker_suffix)
        if use_dotnet_cli:
            self.name += "_dotnetcli"
            self.script_suffix = "_dotnetcli"
            self.labels.append("dotnetcli")
        else:
            self.labels.append("olddotnet")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as there is little opportunity for parallelizing what's inside the distribtests
        if self.platform == "linux":
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/distribtest/csharp_%s_%s"
                % (self.docker_suffix, self.arch),
                "test/distrib/csharp/run_distrib_test%s.sh"
                % self.script_suffix,
                copy_rel_path="test/distrib",
            )
        elif self.platform == "macos":
            return create_jobspec(
                self.name,
                [
                    "test/distrib/csharp/run_distrib_test%s.sh"
                    % self.script_suffix
                ],
                environ={
                    "EXTERNAL_GIT_ROOT": "../../../..",
                    "SKIP_NETCOREAPP21_DISTRIBTEST": "1",
                    "SKIP_NET50_DISTRIBTEST": "1",
                },
                use_workspace=True,
            )
        elif self.platform == "windows":
            # TODO(jtattermusch): re-enable windows distribtest
            return create_jobspec(
                self.name,
                ["bash", "tools/run_tests/artifacts/run_distribtest_csharp.sh"],
                environ={},
                use_workspace=True,
            )
        else:
            raise Exception("Not supported yet.")

    def __str__(self):
        return self.name


class PythonDistribTest:
    """Tests Python package"""

    def __init__(
        self, platform, arch, docker_suffix, source=False, presubmit=False
    ):
        self.source = source
        if source:
            self.name = "python_dev_%s_%s_%s" % (platform, arch, docker_suffix)
        else:
            self.name = "python_%s_%s_%s" % (platform, arch, docker_suffix)
        self.platform = platform
        self.arch = arch
        self.docker_suffix = docker_suffix
        self.labels = ["distribtest", "python", platform, arch, docker_suffix]
        if presubmit:
            self.labels.append("presubmit")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        # TODO(jtattermusch): honor inner_jobs arg for this task.
        del inner_jobs
        if not self.platform == "linux":
            raise Exception("Not supported yet.")

        if self.source:
            # The "source" tests verify that the source distributions can be
            # be installed on popular OSs, see the list in
            # tools/dockerfile/distribtest/python_dev_*.
            #
            # Note that our cython-based packages will be actually compiled
            # during pip install. For this reason:
            #
            # 1. We use python_dev_* images, which contain the build essentials.
            # 2. The source tests will take significantly longer than binary.
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/distribtest/python_dev_%s_%s"
                % (self.docker_suffix, self.arch),
                "test/distrib/python/run_source_distrib_test.sh",
                copy_rel_path="test/distrib",
                timeout_seconds=datetime.timedelta(
                    hours=1,
                    minutes=30,
                ).total_seconds(),
            )
        else:
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/distribtest/python_%s_%s"
                % (self.docker_suffix, self.arch),
                "test/distrib/python/run_binary_distrib_test.sh",
                copy_rel_path="test/distrib",
                # TODO(sergiitk): consider decreasing this, they seem to take
                # only around 5 minutes.
                timeout_seconds=datetime.timedelta(minutes=45).total_seconds(),
            )

    def __str__(self):
        return self.name


class RubyDistribTest:
    """Tests Ruby package"""

    def __init__(
        self,
        platform,
        arch,
        docker_suffix,
        ruby_version=None,
        source=False,
        presubmit=False,
        protobuf_version="",
    ):
        self.package_type = "binary"
        if source:
            self.package_type = "source"
        self.name = "ruby_%s_%s_%s_version_%s_package_type_%s" % (
            platform,
            arch,
            docker_suffix,
            ruby_version or "unspecified",
            self.package_type,
        )
        if not protobuf_version == "":
            self.name += "_protobuf_%s" % protobuf_version
        self.platform = platform
        platform_label = self.platform
        self.arch = arch
        self.docker_suffix = docker_suffix
        self.ruby_version = ruby_version
        self.protobuf_version = protobuf_version
        if platform_label.startswith("linux"):
            platform_label = "linux"
        self.labels = [
            "distribtest",
            "ruby",
            platform_label,
            arch,
            docker_suffix,
        ]
        if presubmit:
            self.labels.append("presubmit")

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        # TODO(jtattermusch): honor inner_jobs arg for this task.
        del inner_jobs
        arch_to_gem_arch = {
            "x64": "x86_64",
            "x86": "x86",
        }
        if self.platform not in ["linux-gnu", "linux-musl"]:
            raise Exception("Not supported yet.")

        dockerfile_name = "tools/dockerfile/distribtest/ruby_%s_%s" % (
            self.docker_suffix,
            self.arch,
        )
        if self.ruby_version is not None:
            dockerfile_name += "_%s" % self.ruby_version
        return create_docker_jobspec(
            self.name,
            dockerfile_name,
            "test/distrib/ruby/run_distrib_test.sh %s %s %s %s"
            % (
                arch_to_gem_arch[self.arch],
                self.platform,
                self.package_type,
                self.protobuf_version,
            ),
            copy_rel_path="test/distrib",
        )

    def __str__(self):
        return self.name


class PHP8DistribTest:
    """Tests PHP8 package"""

    def __init__(self, platform, arch, docker_suffix=None, presubmit=False):
        self.name = "php8_%s_%s_%s" % (platform, arch, docker_suffix)
        self.platform = platform
        self.arch = arch
        self.docker_suffix = docker_suffix
        self.labels = ["distribtest", "php", "php8", platform, arch]
        if presubmit:
            self.labels.append("presubmit")
        if docker_suffix:
            self.labels.append(docker_suffix)

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        # TODO(jtattermusch): honor inner_jobs arg for this task.
        del inner_jobs
        if self.platform == "linux":
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/distribtest/php8_%s_%s"
                % (self.docker_suffix, self.arch),
                "test/distrib/php/run_distrib_test.sh",
                copy_rel_path="test/distrib",
            )
        elif self.platform == "macos":
            return create_jobspec(
                self.name,
                ["test/distrib/php/run_distrib_test_macos.sh"],
                environ={"EXTERNAL_GIT_ROOT": "../../../.."},
                timeout_seconds=30 * 60,
                use_workspace=True,
            )
        else:
            raise Exception("Not supported yet.")

    def __str__(self):
        return self.name


class CppDistribTest:
    """Tests Cpp make install by building examples."""

    def __init__(
        self, platform, arch, docker_suffix=None, testcase=None, presubmit=False
    ):
        if platform == "linux":
            self.name = "cpp_%s_%s_%s_%s" % (
                platform,
                arch,
                docker_suffix,
                testcase,
            )
        else:
            self.name = "cpp_%s_%s_%s" % (platform, arch, testcase)
        self.platform = platform
        self.arch = arch
        self.docker_suffix = docker_suffix
        self.testcase = testcase
        self.labels = [
            "distribtest",
            "cpp",
            platform,
            arch,
            testcase,
        ]
        if presubmit:
            self.labels.append("presubmit")
        if docker_suffix:
            self.labels.append(docker_suffix)

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        environ = {}
        if inner_jobs is not None:
            # set number of parallel jobs for the C++ build
            environ["GRPC_CPP_DISTRIBTEST_BUILD_COMPILER_JOBS"] = str(
                inner_jobs
            )

        if self.platform == "linux":
            return create_docker_jobspec(
                self.name,
                "tools/dockerfile/distribtest/cpp_%s_%s"
                % (self.docker_suffix, self.arch),
                "test/distrib/cpp/run_distrib_test_%s.sh" % self.testcase,
                timeout_seconds=2 * 60 * 60,
            )
        elif self.platform == "windows":
            return create_jobspec(
                self.name,
                ["test\\distrib\\cpp\\run_distrib_test_%s.bat" % self.testcase],
                environ={},
                timeout_seconds=2 * 60 * 60,
                use_workspace=True,
            )
        else:
            raise Exception("Not supported yet.")

    def __str__(self):
        return self.name


def targets():
    """Gets list of supported targets"""
    return [
        # C++
        # The "dummy" C++ distribtest so that the set of tasks to run isn't empty
        # when grpc_distribtest_standalone runs on PRs.
        CppDistribTest("linux", "x64", "debian11", "dummy", presubmit=True),
        CppDistribTest("linux", "x64", "debian11", "cmake", presubmit=False),
        CppDistribTest(
            "linux", "x64", "debian11", "cmake_as_submodule", presubmit=False
        ),
        CppDistribTest(
            "linux",
            "x64",
            "debian11",
            "cmake_as_externalproject",
            presubmit=False,
        ),
        CppDistribTest(
            "linux", "x64", "debian11", "cmake_fetchcontent", presubmit=False
        ),
        CppDistribTest(
            "linux", "x64", "debian11", "cmake_module_install", presubmit=False
        ),
        CppDistribTest(
            "linux", "x64", "debian11", "cmake_pkgconfig", presubmit=False
        ),
        CppDistribTest(
            "linux",
            "x64",
            "debian11_aarch64_cross",
            "cmake_aarch64_cross",
            presubmit=False,
        ),
        CppDistribTest("windows", "x86", testcase="cmake", presubmit=True),
        CppDistribTest(
            "windows",
            "x86",
            testcase="cmake_as_externalproject",
            presubmit=True,
        ),
        CppDistribTest(
            "windows",
            "x86",
            testcase="cmake_for_dll",
            presubmit=True,
        ),
        # C#
        CSharpDistribTest(
            "linux", "x64", "debian11", use_dotnet_cli=True, presubmit=True
        ),
        CSharpDistribTest("linux", "x64", "ubuntu2204", use_dotnet_cli=True),
        CSharpDistribTest(
            "linux", "x64", "alpine", use_dotnet_cli=True, presubmit=True
        ),
        CSharpDistribTest(
            "linux", "x64", "dotnet31", use_dotnet_cli=True, presubmit=True
        ),
        CSharpDistribTest(
            "linux", "x64", "dotnet5", use_dotnet_cli=True, presubmit=True
        ),
        CSharpDistribTest("macos", "x64", use_dotnet_cli=True, presubmit=True),
        CSharpDistribTest("windows", "x86", presubmit=True),
        CSharpDistribTest("windows", "x64", presubmit=True),
        # Python
        PythonDistribTest("linux", "x64", "bullseye", presubmit=True),
        PythonDistribTest("linux", "x86", "bullseye", presubmit=True),
        PythonDistribTest("linux", "x64", "fedora40"),
        PythonDistribTest("linux", "x64", "arch"),
        PythonDistribTest("linux", "x64", "alpine"),
        PythonDistribTest("linux", "x64", "ubuntu2404"),
        PythonDistribTest(
            "linux", "aarch64", "python310_bullseye", presubmit=True
        ),
        PythonDistribTest("linux", "aarch64", "alpine", presubmit=True),
        PythonDistribTest(
            "linux", "x64", "alpine3.18", source=True, presubmit=True
        ),
        PythonDistribTest(
            "linux", "x64", "bullseye", source=True, presubmit=True
        ),
        PythonDistribTest(
            "linux", "x86", "bullseye", source=True, presubmit=True
        ),
        PythonDistribTest("linux", "x64", "fedora40", source=True),
        PythonDistribTest("linux", "x64", "arch", source=True),
        PythonDistribTest("linux", "x64", "ubuntu2404", source=True),
        # Ruby
        RubyDistribTest(
            "linux-gnu",
            "x64",
            "debian11",
            ruby_version="ruby_3_2",
            source=True,
        ),
        RubyDistribTest(
            "linux-gnu",
            "x64",
            "debian11",
            ruby_version="ruby_3_2",
        ),
        RubyDistribTest(
            "linux-gnu",
            "x64",
            "debian11",
            ruby_version="ruby_3_3",
        ),
        RubyDistribTest(
            "linux-gnu",
            "x64",
            "debian11",
            ruby_version="ruby_3_3",
            protobuf_version="3.25",
        ),
        RubyDistribTest(
            "linux-gnu",
            "x64",
            "debian11",
            ruby_version="ruby_3_4",
        ),
        RubyDistribTest(
            "linux-gnu",
            "x64",
            "debian11",
            ruby_version="ruby_4_0",
            presubmit=True,
        ),
        RubyDistribTest("linux-gnu", "x64", "ubuntu2204", presubmit=True),
        RubyDistribTest("linux-gnu", "x64", "ubuntu2404", presubmit=True),
        RubyDistribTest(
            "linux-musl",
            "x64",
            "alpine",
            ruby_version="ruby_3_2",
        ),
        RubyDistribTest(
            "linux-musl",
            "x64",
            "alpine",
            ruby_version="ruby_3_3",
        ),
        RubyDistribTest(
            "linux-musl",
            "x64",
            "alpine",
            ruby_version="ruby_3_4",
        ),
        RubyDistribTest(
            "linux-musl",
            "x64",
            "alpine",
            ruby_version="ruby_4_0",
            presubmit=True,
        ),
        # PHP8
        PHP8DistribTest("linux", "x64", "debian12", presubmit=True),
        PHP8DistribTest("macos", "x64", presubmit=True),
    ]



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Send multiple greeting messages to the backend."""

import argparse
import asyncio
import logging

import grpc

helloworld_pb2, helloworld_pb2_grpc = grpc.protos_and_services(
    "helloworld.proto"
)


async def process(
    stub: helloworld_pb2_grpc.GreeterStub, request: helloworld_pb2.HelloRequest
) -> None:
    try:
        response = await stub.SayHello(request)
    except grpc.aio.AioRpcError as rpc_error:
        print(f"Received error: {rpc_error}")
    else:
        print(f"Received message: {response}")


async def run(addr: str, n: int) -> None:
    async with grpc.aio.insecure_channel(addr) as channel:
        stub = helloworld_pb2_grpc.GreeterStub(channel)
        request = helloworld_pb2.HelloRequest(name="you")
        for _ in range(n):
            await process(stub, request)


async def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--addr",
        nargs=1,
        type=str,
        default="[::]:50051",
        help="the address to request",
    )
    parser.add_argument(
        "-n",
        nargs=1,
        type=int,
        default=10,
        help="an integer for number of messages to sent",
    )
    args = parser.parse_args()
    await run(addr=args.addr, n=args.n)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    asyncio.get_event_loop().run_until_complete(main())



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test times."""

import collections
import logging
import threading
import time as _time

import grpc
import grpc_testing

logging.basicConfig()
_LOGGER = logging.getLogger(__name__)


def _call(behaviors):
    for behavior in behaviors:
        try:
            behavior()
        except Exception:  # pylint: disable=broad-except
            _LOGGER.exception('Exception calling behavior "%r"!', behavior)


def _call_in_thread(behaviors):
    calling = threading.Thread(target=_call, args=(behaviors,))
    calling.start()
    # NOTE(nathaniel): Because this function is called from "strict" Time
    # implementations, it blocks until after all behaviors have terminated.
    calling.join()


class _State:
    def __init__(self):
        self.condition = threading.Condition()
        self.times_to_behaviors = collections.defaultdict(list)


class _Delta(
    collections.namedtuple(
        "_Delta",
        (
            "mature_behaviors",
            "earliest_mature_time",
            "earliest_immature_time",
        ),
    )
):
    pass


def _process(state, now):
    mature_behaviors = []
    earliest_mature_time = None
    while state.times_to_behaviors:
        earliest_time = min(state.times_to_behaviors)
        if earliest_time <= now:
            if earliest_mature_time is None:
                earliest_mature_time = earliest_time
            earliest_mature_behaviors = state.times_to_behaviors.pop(
                earliest_time
            )
            mature_behaviors.extend(earliest_mature_behaviors)
        else:
            earliest_immature_time = earliest_time
            break
    else:
        earliest_immature_time = None
    return _Delta(
        mature_behaviors, earliest_mature_time, earliest_immature_time
    )


class _Future(grpc.Future):
    def __init__(self, state, behavior, time):
        self._state = state
        self._behavior = behavior
        self._time = time
        self._cancelled = False

    def cancel(self):
        with self._state.condition:
            if self._cancelled:
                return True
            behaviors_at_time = self._state.times_to_behaviors.get(self._time)
            if behaviors_at_time is None:
                return False
            behaviors_at_time.remove(self._behavior)
            if not behaviors_at_time:
                self._state.times_to_behaviors.pop(self._time)
                self._state.condition.notify_all()
            self._cancelled = True
            return True

    def cancelled(self):
        with self._state.condition:
            return self._cancelled

    def running(self):
        raise NotImplementedError()

    def done(self):
        raise NotImplementedError()

    def result(self, timeout=None):
        raise NotImplementedError()

    def exception(self, timeout=None):
        raise NotImplementedError()

    def traceback(self, timeout=None):
        raise NotImplementedError()

    def add_done_callback(self, fn):
        raise NotImplementedError()


class StrictRealTime(grpc_testing.Time):
    def __init__(self):
        self._state = _State()
        self._active = False
        self._calling = None

    def _activity(self):
        while True:
            with self._state.condition:
                while True:
                    now = _time.time()
                    delta = _process(self._state, now)
                    self._state.condition.notify_all()
                    if delta.mature_behaviors:
                        self._calling = delta.earliest_mature_time
                        break
                    self._calling = None
                    if delta.earliest_immature_time is None:
                        self._active = False
                        return
                    timeout = max(0, delta.earliest_immature_time - now)
                    self._state.condition.wait(timeout=timeout)
            _call(delta.mature_behaviors)

    def _ensure_called_through(self, time):
        with self._state.condition:
            while (
                self._state.times_to_behaviors
                and min(self._state.times_to_behaviors) < time
            ) or (self._calling is not None and self._calling < time):
                self._state.condition.wait()

    def _call_at(self, behavior, time):
        with self._state.condition:
            self._state.times_to_behaviors[time].append(behavior)
            if self._active:
                self._state.condition.notify_all()
            else:
                activity = threading.Thread(target=self._activity)
                activity.start()
                self._active = True
            return _Future(self._state, behavior, time)

    def time(self):
        return _time.time()

    def call_in(self, behavior, delay):
        return self._call_at(behavior, _time.time() + delay)

    def call_at(self, behavior, time):
        return self._call_at(behavior, time)

    def sleep_for(self, duration):
        time = _time.time() + duration
        _time.sleep(duration)
        self._ensure_called_through(time)

    def sleep_until(self, time):
        _time.sleep(max(0, time - _time.time()))
        self._ensure_called_through(time)


class StrictFakeTime(grpc_testing.Time):
    def __init__(self, time):
        self._state = _State()
        self._time = time

    def time(self):
        return self._time

    def call_in(self, behavior, delay):
        if delay <= 0:
            _call_in_thread((behavior,))
        else:
            with self._state.condition:
                time = self._time + delay
                self._state.times_to_behaviors[time].append(behavior)
        return _Future(self._state, behavior, time)

    def call_at(self, behavior, time):
        with self._state.condition:
            if time <= self._time:
                _call_in_thread((behavior,))
            else:
                self._state.times_to_behaviors[time].append(behavior)
        return _Future(self._state, behavior, time)

    def sleep_for(self, duration):
        if duration > 0:
            with self._state.condition:
                self._time += duration
                delta = _process(self._state, self._time)
                _call_in_thread(delta.mature_behaviors)

    def sleep_until(self, time):
        with self._state.condition:
            if self._time < time:
                self._time = time
                delta = _process(self._state, self._time)
                _call_in_thread(delta.mature_behaviors)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/rocketmq_proxy/v3/rocketmq_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/rocketmq_proxy/v3/rocketmq_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from contrib.envoy.extensions.filters.network.rocketmq_proxy.v3 import route_pb2 as contrib_dot_envoy_dot_extensions_dot_filters_dot_network_dot_rocketmq__proxy_dot_v3_dot_route__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nOcontrib/envoy/extensions/filters/network/rocketmq_proxy/v3/rocketmq_proxy.proto\x12\x32\x65nvoy.extensions.filters.network.rocketmq_proxy.v3\x1a\x46\x63ontrib/envoy/extensions/filters/network/rocketmq_proxy/v3/route.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe0\x01\n\rRocketmqProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\\\n\x0croute_config\x18\x02 \x01(\x0b\x32\x46.envoy.extensions.filters.network.rocketmq_proxy.v3.RouteConfiguration\x12=\n\x1atransient_object_life_span\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x14\n\x0c\x64\x65velop_mode\x18\x04 \x01(\x08\x42\xd4\x01\n@io.envoyproxy.envoy.extensions.filters.network.rocketmq_proxy.v3B\x12RocketmqProxyProtoP\x01Zrgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/rocketmq_proxy/v3;rocketmq_proxyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.rocketmq_proxy.v3.rocketmq_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.filters.network.rocketmq_proxy.v3B\022RocketmqProxyProtoP\001Zrgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/rocketmq_proxy/v3;rocketmq_proxyv3\272\200\310\321\006\002\020\002'
  _globals['_ROCKETMQPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_ROCKETMQPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ROCKETMQPROXY']._serialized_start=296
  _globals['_ROCKETMQPROXY']._serialized_end=520
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc.channel_ready_future."""

import logging
import threading
import unittest

import grpc

from tests.unit import thread_pool
from tests.unit.framework.common import test_constants


class _Callback:
    def __init__(self):
        self._condition = threading.Condition()
        self._value = None

    def accept_value(self, value):
        with self._condition:
            self._value = value
            self._condition.notify_all()

    def block_until_called(self):
        with self._condition:
            while self._value is None:
                self._condition.wait()
            return self._value


class ChannelReadyFutureTest(unittest.TestCase):
    def test_lonely_channel_connectivity(self):
        channel = grpc.insecure_channel("localhost:12345")
        callback = _Callback()

        ready_future = grpc.channel_ready_future(channel)
        ready_future.add_done_callback(callback.accept_value)
        with self.assertRaises(grpc.FutureTimeoutError):
            ready_future.result(timeout=test_constants.SHORT_TIMEOUT)
        self.assertFalse(ready_future.cancelled())
        self.assertFalse(ready_future.done())
        self.assertTrue(ready_future.running())
        ready_future.cancel()
        value_passed_to_callback = callback.block_until_called()
        self.assertIs(ready_future, value_passed_to_callback)
        self.assertTrue(ready_future.cancelled())
        self.assertTrue(ready_future.done())
        self.assertFalse(ready_future.running())

        channel.close()

    def test_immediately_connectable_channel_connectivity(self):
        recording_thread_pool = thread_pool.RecordingThreadPool(
            max_workers=None
        )
        server = grpc.server(
            recording_thread_pool, options=(("grpc.so_reuseport", 0),)
        )
        port = server.add_insecure_port("[::]:0")
        server.start()
        channel = grpc.insecure_channel("localhost:{}".format(port))
        callback = _Callback()

        ready_future = grpc.channel_ready_future(channel)
        ready_future.add_done_callback(callback.accept_value)
        self.assertIsNone(
            ready_future.result(timeout=test_constants.LONG_TIMEOUT)
        )
        value_passed_to_callback = callback.block_until_called()
        self.assertIs(ready_future, value_passed_to_callback)
        self.assertFalse(ready_future.cancelled())
        self.assertTrue(ready_future.done())
        self.assertFalse(ready_future.running())
        # Cancellation after maturity has no effect.
        ready_future.cancel()
        self.assertFalse(ready_future.cancelled())
        self.assertTrue(ready_future.done())
        self.assertFalse(ready_future.running())
        self.assertFalse(recording_thread_pool.was_used())

        channel.close()
        server.stop(None)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test servers at the level of the Cython API."""

import threading
import time
import unittest

from grpc._cython import cygrpc


class Test(unittest.TestCase):
    def test_lonely_server(self):
        server_call_completion_queue = cygrpc.CompletionQueue()
        server_shutdown_completion_queue = cygrpc.CompletionQueue()
        server = cygrpc.Server(None, False)
        server.register_completion_queue(server_call_completion_queue)
        server.register_completion_queue(server_shutdown_completion_queue)
        port = server.add_http2_port(b"[::]:0")
        server.start()

        server_request_call_tag = "server_request_call_tag"
        server_request_call_start_batch_result = server.request_call(
            server_call_completion_queue,
            server_call_completion_queue,
            server_request_call_tag,
        )

        time.sleep(4)

        server_shutdown_tag = "server_shutdown_tag"
        server_shutdown_result = server.shutdown(
            server_shutdown_completion_queue, server_shutdown_tag
        )
        server_request_call_event = server_call_completion_queue.poll()
        server_shutdown_event = server_shutdown_completion_queue.poll()


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import concurrent.futures
import queue
import threading
import unittest

import grpc
from grpc.experimental import aio

from tests_aio.unit._test_base import AioTestBase


class GenericService:
    @staticmethod
    async def UnaryCall(request, context):
        return request


class MultithreadTest(AioTestBase):
    async def _start_server(self):
        server = grpc.aio.server()
        rpc_method_handlers = {
            "UnaryCall": grpc.unary_unary_rpc_method_handler(
                GenericService.UnaryCall,
            ),
        }
        generic_handler = grpc.method_handlers_generic_handler(
            "grpc.testing.TestService", rpc_method_handlers
        )
        server.add_generic_rpc_handlers((generic_handler,))
        port = server.add_insecure_port("[::]:0")
        await server.start()
        return port, server

    async def run_client(self, port):
        async with aio.insecure_channel(f"localhost:{port}") as channel:
            unary_call = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall"
            )
            response = await unary_call(b"request")
            return response

    def thread_target(self, port, queue):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            response = loop.run_until_complete(self.run_client(port))
            queue.put(response)
        except Exception as e:
            queue.put(e)
        finally:
            loop.run_until_complete(loop.shutdown_asyncgens())
            loop.close()

    async def _test_multithread(self, executor=None):
        results_queue = queue.Queue()
        port, server = await self._start_server()
        threads = []
        for _ in range(10):
            t = threading.Thread(
                target=self.thread_target, args=(port, results_queue)
            )
            t.start()
            threads.append(t)

        def join_threads():
            for t in threads:
                t.join()

        await self.loop.run_in_executor(executor, join_threads)

        await server.stop(None)

        # Verify results
        self.assertEqual(results_queue.qsize(), 10)
        while not results_queue.empty():
            result = results_queue.get_nowait()
            self.assertIsInstance(result, bytes)
            self.assertEqual(result, b"request")

    async def test_multithread(self):
        await self._test_multithread(executor=None)

    async def test_multithread_with_thread_pool(self):
        with concurrent.futures.ThreadPoolExecutor() as pool:
            await self._test_multithread(executor=pool)


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Helper script to concatenate YAML files.
#
# This script concatenates multiple YAML files into a single multipart file.
# Input files are not parsed but processed as strings. This is a convenience
# script to concatenate output files generated by multiple runs of
# loadtest_config.py.

import argparse
import sys
from typing import Iterable


def gen_content_strings(input_files: Iterable[str]) -> Iterable[str]:
    if not input_files:
        return

    with open(input_files[0]) as f:
        content = f.read()
    yield content

    for input_file in input_files[1:]:
        with open(input_file) as f:
            content = f.read()
        yield "---\n"
        yield content


def main() -> None:
    argp = argparse.ArgumentParser(description="Concatenates YAML files.")
    argp.add_argument(
        "-i",
        "--inputs",
        action="extend",
        nargs="+",
        type=str,
        required=True,
        help="Input files.",
    )
    argp.add_argument(
        "-o",
        "--output",
        type=str,
        help="Concatenated output file. Output to stdout if not set.",
    )
    args = argp.parse_args()

    with open(args.output, "w") if args.output else sys.stdout as f:
        for content in gen_content_strings(args.inputs):
            print(content, file=f, sep="", end="")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: route_guide.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11route_guide.proto\x12\nrouteguide\",\n\x05Point\x12\x10\n\x08latitude\x18\x01 \x01(\x05\x12\x11\n\tlongitude\x18\x02 \x01(\x05\"I\n\tRectangle\x12\x1d\n\x02lo\x18\x01 \x01(\x0b\x32\x11.routeguide.Point\x12\x1d\n\x02hi\x18\x02 \x01(\x0b\x32\x11.routeguide.Point\"<\n\x07\x46\x65\x61ture\x12\x0c\n\x04name\x18\x01 \x01(\t\x12#\n\x08location\x18\x02 \x01(\x0b\x32\x11.routeguide.Point\"A\n\tRouteNote\x12#\n\x08location\x18\x01 \x01(\x0b\x32\x11.routeguide.Point\x12\x0f\n\x07message\x18\x02 \x01(\t\"b\n\x0cRouteSummary\x12\x13\n\x0bpoint_count\x18\x01 \x01(\x05\x12\x15\n\rfeature_count\x18\x02 \x01(\x05\x12\x10\n\x08\x64istance\x18\x03 \x01(\x05\x12\x14\n\x0c\x65lapsed_time\x18\x04 \x01(\x05\x32\x85\x02\n\nRouteGuide\x12\x36\n\nGetFeature\x12\x11.routeguide.Point\x1a\x13.routeguide.Feature\"\x00\x12>\n\x0cListFeatures\x12\x15.routeguide.Rectangle\x1a\x13.routeguide.Feature\"\x00\x30\x01\x12>\n\x0bRecordRoute\x12\x11.routeguide.Point\x1a\x18.routeguide.RouteSummary\"\x00(\x01\x12?\n\tRouteChat\x12\x15.routeguide.RouteNote\x1a\x15.routeguide.RouteNote\"\x00(\x01\x30\x01\x42\x36\n\x1bio.grpc.examples.routeguideB\x0fRouteGuideProtoP\x01\xa2\x02\x03RTGb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'route_guide_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033io.grpc.examples.routeguideB\017RouteGuideProtoP\001\242\002\003RTG'
  _POINT._serialized_start=33
  _POINT._serialized_end=77
  _RECTANGLE._serialized_start=79
  _RECTANGLE._serialized_end=152
  _FEATURE._serialized_start=154
  _FEATURE._serialized_end=214
  _ROUTENOTE._serialized_start=216
  _ROUTENOTE._serialized_end=281
  _ROUTESUMMARY._serialized_start=283
  _ROUTESUMMARY._serialized_end=381
  _ROUTEGUIDE._serialized_start=384
  _ROUTEGUIDE._serialized_end=645
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from envoy.service.trace.v3 import trace_service_pb2 as envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2


class TraceServiceStub:
    """[#protodoc-title: Trace service]

    Service for streaming traces to server that consumes the trace data. It
    uses OpenCensus data model as a standard to represent trace information.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StreamTraces = channel.stream_unary(
                '/envoy.service.trace.v3.TraceService/StreamTraces',
                request_serializer=envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2.StreamTracesMessage.SerializeToString,
                response_deserializer=envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2.StreamTracesResponse.FromString,
                )


class TraceServiceServicer:
    """[#protodoc-title: Trace service]

    Service for streaming traces to server that consumes the trace data. It
    uses OpenCensus data model as a standard to represent trace information.
    """

    def StreamTraces(self, request_iterator, context):
        """Envoy will connect and send StreamTracesMessage messages forever. It does
        not expect any response to be sent as nothing would be done in the case
        of failure.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TraceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StreamTraces': grpc.stream_unary_rpc_method_handler(
                    servicer.StreamTraces,
                    request_deserializer=envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2.StreamTracesMessage.FromString,
                    response_serializer=envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2.StreamTracesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'envoy.service.trace.v3.TraceService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TraceService:
    """[#protodoc-title: Trace service]

    Service for streaming traces to server that consumes the trace data. It
    uses OpenCensus data model as a standard to represent trace information.
    """

    @staticmethod
    def StreamTraces(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/envoy.service.trace.v3.TraceService/StreamTraces',
            envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2.StreamTracesMessage.SerializeToString,
            envoy_dot_service_dot_trace_dot_v3_dot_trace__service__pb2.StreamTracesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Testing the compatibility between AsyncIO stack and the old stack."""

import asyncio
from concurrent.futures import ThreadPoolExecutor
import logging
import os
import random
import threading
from typing import Callable, Iterable, Sequence, Tuple
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework.common import test_constants
from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import TestServiceServicer
from tests_aio.unit._test_server import start_test_server

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 42
_REQUEST = b"\x03\x07"


def _unique_options() -> Sequence[Tuple[str, float]]:
    return (("iv", random.random()),)


@unittest.skipIf(
    os.environ.get("GRPC_ASYNCIO_ENGINE", "").lower() == "custom_io_manager",
    "Compatible mode needs POLLER completion queue.",
)
class TestCompatibility(AioTestBase):
    async def setUp(self):
        self._async_server = aio.server(
            options=(("grpc.so_reuseport", 0),),
            migration_thread_pool=ThreadPoolExecutor(),
        )

        test_pb2_grpc.add_TestServiceServicer_to_server(
            TestServiceServicer(), self._async_server
        )
        self._adhoc_handlers = _common.AdhocGenericHandler()
        self._async_server.add_generic_rpc_handlers((self._adhoc_handlers,))

        port = self._async_server.add_insecure_port("[::]:0")
        address = "localhost:%d" % port
        await self._async_server.start()

        # Create async stub
        self._async_channel = aio.insecure_channel(
            address, options=_unique_options()
        )
        self._async_stub = test_pb2_grpc.TestServiceStub(self._async_channel)

        # Create sync stub
        self._sync_channel = grpc.insecure_channel(
            address, options=_unique_options()
        )
        self._sync_stub = test_pb2_grpc.TestServiceStub(self._sync_channel)

    async def tearDown(self):
        self._sync_channel.close()
        await self._async_channel.close()
        await self._async_server.stop(None)

    async def _run_in_another_thread(self, func: Callable[[], None]):
        work_done = asyncio.Event()

        def thread_work():
            func()
            self.loop.call_soon_threadsafe(work_done.set)

        thread = threading.Thread(target=thread_work, daemon=True)
        thread.start()
        await work_done.wait()
        thread.join()

    async def test_unary_unary(self):
        # Calling async API in this thread
        await self._async_stub.UnaryCall(
            messages_pb2.SimpleRequest(), timeout=test_constants.LONG_TIMEOUT
        )

        # Calling sync API in a different thread
        def sync_work() -> None:
            response, call = self._sync_stub.UnaryCall.with_call(
                messages_pb2.SimpleRequest(),
                timeout=test_constants.LONG_TIMEOUT,
            )
            self.assertIsInstance(response, messages_pb2.SimpleResponse)
            self.assertEqual(grpc.StatusCode.OK, call.code())

        await self._run_in_another_thread(sync_work)

    async def test_unary_stream(self):
        request = messages_pb2.StreamingOutputCallRequest()
        for _ in range(_NUM_STREAM_RESPONSES):
            request.response_parameters.append(
                messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
            )

        # Calling async API in this thread
        call = self._async_stub.StreamingOutputCall(request)

        for _ in range(_NUM_STREAM_RESPONSES):
            await call.read()
        self.assertEqual(grpc.StatusCode.OK, await call.code())

        # Calling sync API in a different thread
        def sync_work() -> None:
            response_iterator = self._sync_stub.StreamingOutputCall(request)
            for response in response_iterator:
                assert _RESPONSE_PAYLOAD_SIZE == len(response.payload.body)
            self.assertEqual(grpc.StatusCode.OK, response_iterator.code())

        await self._run_in_another_thread(sync_work)

    async def test_stream_unary(self):
        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        # Calling async API in this thread
        async def gen():
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request

        response = await self._async_stub.StreamingInputCall(gen())
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )

        # Calling sync API in a different thread
        def sync_work() -> None:
            response = self._sync_stub.StreamingInputCall(
                iter([request] * _NUM_STREAM_RESPONSES)
            )
            self.assertEqual(
                _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
                response.aggregated_payload_size,
            )

        await self._run_in_another_thread(sync_work)

    async def test_stream_stream(self):
        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.append(
            messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
        )

        # Calling async API in this thread
        call = self._async_stub.FullDuplexCall()

        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
            response = await call.read()
            assert _RESPONSE_PAYLOAD_SIZE == len(response.payload.body)

        await call.done_writing()
        assert await call.code() == grpc.StatusCode.OK

        # Calling sync API in a different thread
        def sync_work() -> None:
            response_iterator = self._sync_stub.FullDuplexCall(iter([request]))
            for response in response_iterator:
                assert _RESPONSE_PAYLOAD_SIZE == len(response.payload.body)
            self.assertEqual(grpc.StatusCode.OK, response_iterator.code())

        await self._run_in_another_thread(sync_work)

    async def test_server(self):
        class GenericHandlers(grpc.GenericRpcHandler):
            def service(self, handler_call_details):
                return grpc.unary_unary_rpc_method_handler(lambda x, _: x)

        # It's fine to instantiate server object in the event loop thread.
        # The server will spawn its own serving thread.
        server = grpc.server(
            ThreadPoolExecutor(), handlers=(GenericHandlers(),)
        )
        port = server.add_insecure_port("localhost:0")
        server.start()

        def sync_work() -> None:
            for _ in range(100):
                with grpc.insecure_channel("localhost:%d" % port) as channel:
                    response = channel.unary_unary("/test/test")(b"\x07\x08")
                    self.assertEqual(response, b"\x07\x08")

        await self._run_in_another_thread(sync_work)

    async def test_many_loop(self):
        address, server = await start_test_server()

        # Run another loop in another thread
        def sync_work():
            async def async_work():
                # Create async stub
                async_channel = aio.insecure_channel(
                    address, options=_unique_options()
                )
                async_stub = test_pb2_grpc.TestServiceStub(async_channel)

                call = async_stub.UnaryCall(messages_pb2.SimpleRequest())
                response = await call
                self.assertIsInstance(response, messages_pb2.SimpleResponse)
                self.assertEqual(grpc.StatusCode.OK, await call.code())

            loop = asyncio.new_event_loop()
            loop.run_until_complete(async_work())

        await self._run_in_another_thread(sync_work)
        await server.stop(None)

    async def test_sync_unary_unary_success(self):
        @grpc.unary_unary_rpc_method_handler
        def echo_unary_unary(request: bytes, unused_context):
            return request

        self._adhoc_handlers.set_adhoc_handler(echo_unary_unary)
        response = await self._async_channel.unary_unary(_common.ADHOC_METHOD)(
            _REQUEST
        )
        self.assertEqual(_REQUEST, response)

    async def test_sync_unary_unary_metadata(self):
        metadata = (("unique", "key-42"),)

        @grpc.unary_unary_rpc_method_handler
        def metadata_unary_unary(request: bytes, context: grpc.ServicerContext):
            context.send_initial_metadata(metadata)
            return request

        self._adhoc_handlers.set_adhoc_handler(metadata_unary_unary)
        call = self._async_channel.unary_unary(_common.ADHOC_METHOD)(_REQUEST)
        self.assertTrue(
            _common.seen_metadata(
                aio.Metadata(*metadata), await call.initial_metadata()
            )
        )

    async def test_sync_unary_unary_abort(self):
        @grpc.unary_unary_rpc_method_handler
        def abort_unary_unary(request: bytes, context: grpc.ServicerContext):
            context.abort(grpc.StatusCode.INTERNAL, "Test")

        self._adhoc_handlers.set_adhoc_handler(abort_unary_unary)
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._async_channel.unary_unary(_common.ADHOC_METHOD)(
                _REQUEST
            )
        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    async def test_sync_unary_unary_set_code(self):
        @grpc.unary_unary_rpc_method_handler
        def set_code_unary_unary(request: bytes, context: grpc.ServicerContext):
            context.set_code(grpc.StatusCode.INTERNAL)

        self._adhoc_handlers.set_adhoc_handler(set_code_unary_unary)
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._async_channel.unary_unary(_common.ADHOC_METHOD)(
                _REQUEST
            )
        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    async def test_sync_unary_stream_success(self):
        @grpc.unary_stream_rpc_method_handler
        def echo_unary_stream(request: bytes, unused_context):
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request

        self._adhoc_handlers.set_adhoc_handler(echo_unary_stream)
        call = self._async_channel.unary_stream(_common.ADHOC_METHOD)(_REQUEST)
        async for response in call:
            self.assertEqual(_REQUEST, response)

    async def test_sync_unary_stream_error(self):
        @grpc.unary_stream_rpc_method_handler
        def error_unary_stream(request: bytes, unused_context):
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request
            raise RuntimeError("Test")

        self._adhoc_handlers.set_adhoc_handler(error_unary_stream)
        call = self._async_channel.unary_stream(_common.ADHOC_METHOD)(_REQUEST)
        with self.assertRaises(aio.AioRpcError) as exception_context:
            async for response in call:
                self.assertEqual(_REQUEST, response)
        self.assertEqual(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )

    async def test_sync_stream_unary_success(self):
        @grpc.stream_unary_rpc_method_handler
        def echo_stream_unary(
            request_iterator: Iterable[bytes], unused_context
        ):
            self.assertEqual(len(list(request_iterator)), _NUM_STREAM_RESPONSES)
            return _REQUEST

        self._adhoc_handlers.set_adhoc_handler(echo_stream_unary)
        request_iterator = iter([_REQUEST] * _NUM_STREAM_RESPONSES)
        response = await self._async_channel.stream_unary(_common.ADHOC_METHOD)(
            request_iterator
        )
        self.assertEqual(_REQUEST, response)

    async def test_sync_stream_unary_error(self):
        @grpc.stream_unary_rpc_method_handler
        def echo_stream_unary(
            request_iterator: Iterable[bytes], unused_context
        ):
            self.assertEqual(len(list(request_iterator)), _NUM_STREAM_RESPONSES)
            raise RuntimeError("Test")

        self._adhoc_handlers.set_adhoc_handler(echo_stream_unary)
        request_iterator = iter([_REQUEST] * _NUM_STREAM_RESPONSES)
        with self.assertRaises(aio.AioRpcError) as exception_context:
            response = await self._async_channel.stream_unary(
                _common.ADHOC_METHOD
            )(request_iterator)
        self.assertEqual(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )

    async def test_sync_stream_stream_success(self):
        @grpc.stream_stream_rpc_method_handler
        def echo_stream_stream(
            request_iterator: Iterable[bytes], unused_context
        ):
            for request in request_iterator:
                yield request

        self._adhoc_handlers.set_adhoc_handler(echo_stream_stream)
        request_iterator = iter([_REQUEST] * _NUM_STREAM_RESPONSES)
        call = self._async_channel.stream_stream(_common.ADHOC_METHOD)(
            request_iterator
        )
        async for response in call:
            self.assertEqual(_REQUEST, response)

    async def test_sync_stream_stream_error(self):
        @grpc.stream_stream_rpc_method_handler
        def echo_stream_stream(
            request_iterator: Iterable[bytes], unused_context
        ):
            for request in request_iterator:
                yield request
            raise RuntimeError("test")

        self._adhoc_handlers.set_adhoc_handler(echo_stream_stream)
        request_iterator = iter([_REQUEST] * _NUM_STREAM_RESPONSES)
        call = self._async_channel.stream_stream(_common.ADHOC_METHOD)(
            request_iterator
        )
        with self.assertRaises(aio.AioRpcError) as exception_context:
            async for response in call:
                self.assertEqual(_REQUEST, response)
        self.assertEqual(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.extensions.filters.network.dubbo_proxy.v3 import route_pb2 as envoy_dot_extensions_dot_filters_dot_network_dot_dubbo__proxy_dot_v3_dot_route__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAenvoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto\x12/envoy.extensions.filters.network.dubbo_proxy.v3\x1a(envoy/config/core/v3/config_source.proto\x1a;envoy/extensions/filters/network/dubbo_proxy/v3/route.proto\x1a\x19google/protobuf/any.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"f\n\x04\x44rds\x12\x43\n\rconfig_source\x18\x01 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x19\n\x11route_config_name\x18\x02 \x01(\t\"\xd6\x05\n\nDubboProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12^\n\rprotocol_type\x18\x02 \x01(\x0e\x32=.envoy.extensions.filters.network.dubbo_proxy.v3.ProtocolTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12h\n\x12serialization_type\x18\x03 \x01(\x0e\x32\x42.envoy.extensions.filters.network.dubbo_proxy.v3.SerializationTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x66\n\x0croute_config\x18\x04 \x03(\x0b\x32\x43.envoy.extensions.filters.network.dubbo_proxy.v3.RouteConfigurationB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12^\n\x04\x64rds\x18\x06 \x01(\x0b\x32\x35.envoy.extensions.filters.network.dubbo_proxy.v3.DrdsB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0froute_specifierH\x00\x12l\n\x15multiple_route_config\x18\x07 \x01(\x0b\x32K.envoy.extensions.filters.network.dubbo_proxy.v3.MultipleRouteConfigurationH\x00\x12S\n\rdubbo_filters\x18\x05 \x03(\x0b\x32<.envoy.extensions.filters.network.dubbo_proxy.v3.DubboFilter:B\x9a\xc5\x88\x1e=\n;envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboProxyB\x11\n\x0froute_specifier\"\x8f\x01\n\x0b\x44ubboFilter\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12$\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:C\x9a\xc5\x88\x1e>\n<envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboFilter*\x19\n\x0cProtocolType\x12\t\n\x05\x44ubbo\x10\x00*!\n\x11SerializationType\x12\x0c\n\x08Hessian2\x10\x00\x42\xc0\x01\n=io.envoyproxy.envoy.extensions.filters.network.dubbo_proxy.v3B\x0f\x44ubboProxyProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/dubbo_proxy/v3;dubbo_proxyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.dubbo_proxy.v3.dubbo_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.filters.network.dubbo_proxy.v3B\017DubboProxyProtoP\001Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/dubbo_proxy/v3;dubbo_proxyv3\272\200\310\321\006\002\020\002'
  _globals['_DRDS'].fields_by_name['config_source']._loaded_options = None
  _globals['_DRDS'].fields_by_name['config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['protocol_type']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['protocol_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['serialization_type']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['serialization_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['route_config']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['route_config']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_DUBBOPROXY'].fields_by_name['drds']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['drds']._serialized_options = b'\362\230\376\217\005\021\022\017route_specifier'
  _globals['_DUBBOPROXY']._loaded_options = None
  _globals['_DUBBOPROXY']._serialized_options = b'\232\305\210\036=\n;envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboProxy'
  _globals['_DUBBOFILTER'].fields_by_name['name']._loaded_options = None
  _globals['_DUBBOFILTER'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DUBBOFILTER']._loaded_options = None
  _globals['_DUBBOFILTER']._serialized_options = b'\232\305\210\036>\n<envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboFilter'
  _globals['_PROTOCOLTYPE']._serialized_start=1387
  _globals['_PROTOCOLTYPE']._serialized_end=1412
  _globals['_SERIALIZATIONTYPE']._serialized_start=1414
  _globals['_SERIALIZATIONTYPE']._serialized_end=1447
  _globals['_DRDS']._serialized_start=408
  _globals['_DRDS']._serialized_end=510
  _globals['_DUBBOPROXY']._serialized_start=513
  _globals['_DUBBOPROXY']._serialized_end=1239
  _globals['_DUBBOFILTER']._serialized_start=1242
  _globals['_DUBBOFILTER']._serialized_end=1385
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Setup module for gRPC Python's testing package."""

import os
import sys

import setuptools

# Manually insert the source directory into the Python path for local module
# imports to succeed
sys.path.insert(0, os.path.abspath("."))

# Break import style to ensure that we can find same-directory modules.
import grpc_version
import python_version


class _NoOpCommand(setuptools.Command):
    """No-op command."""

    description = ""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        pass


INSTALL_REQUIRES = (
    "protobuf>=6.33.5,<8.0.0",
    "grpcio>={version}".format(version=grpc_version.VERSION),
)

try:
    import testing_commands as _testing_commands

    # we are in the build environment, otherwise the above import fails
    COMMAND_CLASS = {
        # Run preprocess from the repository *before* doing any packaging!
        "preprocess": _testing_commands.Preprocess,
    }
except ImportError:
    COMMAND_CLASS = {
        # wire up commands to no-op not to break the external dependencies
        "preprocess": _NoOpCommand,
    }

CLASSIFIERS = [
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
]

if __name__ == "__main__":
    setuptools.setup(
        python_requires=f">={python_version.MIN_PYTHON_VERSION}",
        install_requires=INSTALL_REQUIRES,
        cmdclass=COMMAND_CLASS,
        classifiers=CLASSIFIERS,
    )
